# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Queue objects for Barbican
"""
import oslo_messaging as messaging
from oslo_messaging.notify import dispatcher as notify_dispatcher
from oslo_messaging.notify import listener
from oslo_messaging.rpc import dispatcher

from barbican.common import config
from barbican.common import exception


# Constant at one place if this needs to be changed later
KS_NOTIFICATIONS_GRP_NAME = config.KS_NOTIFICATIONS_GRP_NAME

CONF = config.CONF

TRANSPORT = None
IS_SERVER_SIDE = True

ALLOWED_EXMODS = [
    exception.__name__,
]


def get_allowed_exmods():
    return ALLOWED_EXMODS


def init(conf, is_server_side=True):
    global TRANSPORT, IS_SERVER_SIDE
    exmods = get_allowed_exmods()
    IS_SERVER_SIDE = is_server_side
    TRANSPORT = messaging.get_rpc_transport(conf, allowed_remote_exmods=exmods)


def is_server_side():
    return IS_SERVER_SIDE


def cleanup():
    global TRANSPORT
    TRANSPORT.cleanup()
    TRANSPORT = None


def get_target():
    return messaging.Target(topic=CONF.queue.topic,
                            namespace=CONF.queue.namespace,
                            version=CONF.queue.version,
                            server=CONF.queue.server_name)


def get_client(target=None, version_cap=None, serializer=None):
    if not CONF.queue.enable:
        return None

    queue_target = target or get_target()
    return messaging.RPCClient(TRANSPORT,
                               target=queue_target,
                               version_cap=version_cap,
                               serializer=serializer)


def get_server(target, endpoints, serializer=None):
    access_policy = dispatcher.DefaultRPCAccessPolicy
    return messaging.get_rpc_server(TRANSPORT,
                                    target,
                                    endpoints,
                                    executor='eventlet',
                                    serializer=serializer,
                                    access_policy=access_policy)


def get_notification_target():
    conf_opts = getattr(CONF, KS_NOTIFICATIONS_GRP_NAME)
    return messaging.Target(exchange=conf_opts.control_exchange,
                            topic=conf_opts.topic,
                            version=conf_opts.version,
                            fanout=True)


def get_notification_server(targets, endpoints, serializer=None):
    """Retrieve notification server

    This Notification server uses same transport configuration as used by
    other barbican functionality like async order processing.

    Assumption is that messaging infrastructure is going to be shared (same)
    among different barbican features.
    """
    allow_requeue = getattr(getattr(CONF, KS_NOTIFICATIONS_GRP_NAME),
                            'allow_requeue')
    pool_name = getattr(getattr(CONF, KS_NOTIFICATIONS_GRP_NAME),
                        'pool_name')
    TRANSPORT._require_driver_features(requeue=allow_requeue)
    dispatcher = notify_dispatcher.NotificationDispatcher(endpoints,
                                                          serializer)
    # we don't want blocking executor so use eventlet as executor choice
    return listener.NotificationServer(TRANSPORT, targets, dispatcher,
                                       executor='eventlet',
                                       pool=pool_name,
                                       allow_requeue=allow_requeue)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from stevedore import named
import threading

from barbican.common import config
from barbican.common import utils
from barbican import i18n as u
from barbican.plugin.crypto import base
from barbican.plugin.util import multiple_backends
from barbican.plugin.util import utils as plugin_utils


_PLUGIN_MANAGER = None
_PLUGIN_MANAGER_LOCK = threading.RLock()

CONF = config.new_config()

DEFAULT_PLUGIN_NAMESPACE = 'barbican.crypto.plugin'
DEFAULT_PLUGINS = ['simple_crypto']

crypto_opt_group = cfg.OptGroup(name='crypto',
                                title='Crypto Plugin Options')
crypto_opts = [
    cfg.StrOpt('namespace',
               default=DEFAULT_PLUGIN_NAMESPACE,
               help=u._('Extension namespace to search for plugins.')
               ),
    cfg.MultiStrOpt('enabled_crypto_plugins',
                    default=DEFAULT_PLUGINS,
                    help=u._('List of crypto plugins to load.')
                    )
]
CONF.register_group(crypto_opt_group)
CONF.register_opts(crypto_opts, group=crypto_opt_group)
config.parse_args(CONF)

config.set_module_config("crypto", CONF)


def list_opts():
    yield crypto_opt_group, crypto_opts


class _CryptoPluginManager(named.NamedExtensionManager):
    def __init__(self, conf=CONF, invoke_args=(), invoke_kwargs={}):
        """Crypto Plugin Manager

        Each time this class is initialized it will load a new instance
        of each enabled crypto plugin. This is undesirable, so rather than
        initializing a new instance of this class use the PLUGIN_MANAGER
        at the module level.
        """
        crypto_conf = config.get_module_config('crypto')
        plugin_names = self._get_internal_plugin_names(crypto_conf)

        super(_CryptoPluginManager, self).__init__(
            crypto_conf.crypto.namespace,
            plugin_names,
            invoke_on_load=False,  # Defer creating plugins to utility below.
            invoke_args=invoke_args,
            invoke_kwds=invoke_kwargs,
            name_order=True  # extensions sorted as per order of plugin names
        )

        plugin_utils.instantiate_plugins(
            self, invoke_args, invoke_kwargs)

    def get_plugin_store_generate(self, type_needed, algorithm=None,
                                  bit_length=None, mode=None, project_id=None):
        """Gets a secret store or generate plugin that supports provided type.

        :param type_needed: PluginSupportTypes that contains details on the
        type of plugin required
        :returns: CryptoPluginBase plugin implementation
        """
        active_plugins = multiple_backends.get_applicable_crypto_plugins(
            self, project_id=project_id, existing_plugin_name=None)

        if not active_plugins:
            raise base.CryptoPluginNotFound()

        for generating_plugin in active_plugins:
            if generating_plugin.supports(
                    type_needed, algorithm, bit_length, mode):
                break
        else:
            operation = (u._("store or generate a secret of type {secret_type}"
                             " with algorithm {algorithm}, bit length "
                             "{bit_length}, and mode {mode}")
                         .format(secret_type=type_needed,
                                 algorithm=algorithm,
                                 bit_length=bit_length,
                                 mode=mode))
            raise base.CryptoPluginUnsupportedOperation(operation=operation)

        return generating_plugin

    def get_plugin_retrieve(self, plugin_name_for_store):
        """Gets a secret retrieve plugin that supports the provided type.

        :param type_needed: PluginSupportTypes that contains details on the
        type of plugin required
        :returns: CryptoPluginBase plugin implementation
        """
        active_plugins = plugin_utils.get_active_plugins(self)

        if not active_plugins:
            raise base.CryptoPluginNotFound()

        for decrypting_plugin in active_plugins:
            plugin_name = utils.generate_fullname_for(decrypting_plugin)
            if plugin_name == plugin_name_for_store:
                break
        else:
            operation = (u._("retrieve a secret from plugin: {plugin}")
                         .format(plugin=plugin_name_for_store))
            raise base.CryptoPluginUnsupportedOperation(operation=operation)

        return decrypting_plugin

    def _get_internal_plugin_names(self, crypto_conf):
        """Gets plugin names used for loading via stevedore.

        When multiple secret store support is enabled, then crypto plugin names
        are read via updated configuration structure. If not enabled, then it
        reads MultiStr property in 'crypto' config section.
        """
        # to cache default global secret store value on first use
        self.global_default_store_dict = None
        if utils.is_multiple_backends_enabled():
            parsed_stores = multiple_backends.read_multiple_backends_config()
            plugin_names = [store.crypto_plugin for store in parsed_stores
                            if store.crypto_plugin]
        else:
            plugin_names = crypto_conf.crypto.enabled_crypto_plugins
        return plugin_names


def get_manager():
    """Return a singleton crypto plugin manager."""
    global _PLUGIN_MANAGER
    global _PLUGIN_MANAGER_LOCK
    if not _PLUGIN_MANAGER:
        with _PLUGIN_MANAGER_LOCK:
            if not _PLUGIN_MANAGER:
                _PLUGIN_MANAGER = _CryptoPluginManager()
    return _PLUGIN_MANAGER



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import uuid

from barbican.common import config
from barbican.common import exception
from barbican.model import repositories
from barbican.tests.api.controllers import test_secrets as secret_helper
from barbican.tests import utils

containers_repo = repositories.get_container_repository()


class SuccessfulContainerCreateMixin(object):
    def _assert_successful_container_create(self, resp, container_uuid):
        self.assertEqual(201, resp.status_int)
        # this will raise if the container uuid is not proper
        uuid.UUID(container_uuid)


class WhenCreatingContainersUsingContainersResource(
        utils.BarbicanAPIBaseTestCase,
        SuccessfulContainerCreateMixin):

    def test_should_add_new_empty_container(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(container_name, container.name)
        self.assertEqual(container_type, container.type)

    def test_should_add_new_populated_container(self):
        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        container_name = 'test container name'
        container_type = 'generic'
        secret_refs = [
            {
                'name': secret_name,
                'secret_ref': secret_ref
            }
        ]
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type,
            secret_refs=secret_refs
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(container_name, container.name)
        self.assertEqual(container_type, container.type)

    def test_should_create_container_w_empty_name(self):
        # Name key missing
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        # Name key is null
        request = {
            'name': None,
            'type': container_type,
        }
        resp = self.app.post_json(
            '/containers/',
            request,
        )
        container_ref = resp.json.get('container_ref', '')
        _, container_uuid = os.path.split(container_ref)
        self._assert_successful_container_create(resp, container_uuid)

    def test_should_raise_container_bad_json(self):
        resp, container_uuid = create_container(
            self.app,
            expect_errors=True
        )
        self.assertEqual(400, resp.status_int)

    def test_should_raise_container_bad_content_type_header(self):
        resp, container_uuid = create_container(
            self.app,
            name='test container name',
            container_type='generic',
            expect_errors=True,
            headers={'Content-Type': 'bad_content_type'}
        )
        self.assertEqual(415, resp.status_int)

    def test_should_sanitize_location_from_response_header(self):
        resp, container_uuid = create_container(
            self.app,
            name='test container name',
            container_type='generic'
        )
        self._assert_successful_container_create(resp, container_uuid)
        self.assertNotIn(self.project_id, resp.headers['Location'])

    def test_should_throw_exception_when_secret_ref_doesnt_exist(self):
        config.CONF.set_override("host_href", "http://localhost:9311")
        secret_refs = [
            {
                'name': 'bad secret',
                'secret_ref': 'http://localhost:9311/secrets/does_not_exist'
            }
        ]
        resp, container_uuid = create_container(
            self.app,
            name='test container name',
            container_type='generic',
            secret_refs=secret_refs,
            expect_errors=True,
        )
        self.assertEqual(404, resp.status_int)
        config.CONF.clear_override('host_href')


class WhenGettingContainersListUsingContainersResource(
        utils.BarbicanAPIBaseTestCase,
        SuccessfulContainerCreateMixin):

    def setUp(self):
        super(WhenGettingContainersListUsingContainersResource, self).setUp()

        self.num_containers = 10
        self.offset = 2
        self.limit = 2
        self.params = {
            'offset': self.offset,
            'limit': self.limit
        }

    def _create_containers(self, type='generic'):
        for i in range(self.num_containers):
            resp, container_uuid = create_container(
                self.app,
                name='test container name {num}'.format(num=i),
                container_type=type
            )
            self._assert_successful_container_create(resp, container_uuid)

    def _create_url(self, offset_arg=None, limit_arg=None):
        if limit_arg:
            offset = int(offset_arg)
            limit = int(limit_arg)
            return '/containers?limit={limit}&offset={offset}'.format(
                limit=limit, offset=offset)
        else:
            return '/containers'

    def test_should_get_list_containers(self):
        self._create_containers()

        resp = self.app.get(
            '/containers/',
            self.params
        )
        self.assertEqual(200, resp.status_int)
        self.assertIn('previous', resp.namespace)
        self.assertIn('next', resp.namespace)

        url_nav_next = self._create_url(self.offset + self.limit, self.limit)
        self.assertEqual(1, resp.body.decode('utf-8').count(url_nav_next))

        url_nav_prev = self._create_url(0, self.limit)
        self.assertEqual(1, resp.body.decode('utf-8').count(url_nav_prev))

        url_hrefs = self._create_url()
        self.assertEqual((self.limit + 2),
                         resp.body.decode('utf-8').count(url_hrefs))

    def test_list_containerss_by_type(self):
        # Creating containers to be retrieved later
        self._create_containers(type='generic')
        self._create_containers(type='certificate')
        self._create_containers(type='rsa')

        for type in ('generic', 'certificate', 'rsa'):
            params = {
                'limit': self.num_containers,
                'type': type
            }
            resp = self.app.get(
                '/containers/',
                params
            )
            self.assertEqual(200, resp.status_int)
            self.assertEqual(self.num_containers, resp.namespace.get('total'))

    def test_response_should_include_total(self):
        self._create_containers()

        resp = self.app.get(
            '/containers/',
            self.params
        )
        self.assertIn('total', resp.namespace)
        self.assertEqual(self.num_containers, resp.namespace['total'])

    def test_should_handle_no_containers(self):
        resp = self.app.get(
            '/containers/',
            self.params
        )
        self.assertEqual(0, resp.namespace['total'])
        self.assertNotIn('previous', resp.namespace)
        self.assertNotIn('next', resp.namespace)


class WhenGettingOrDeletingContainerUsingContainerResource(
        utils.BarbicanAPIBaseTestCase,
        SuccessfulContainerCreateMixin):

    def test_should_get_container(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp = self.app.get('/containers/{container_id}/'.format(
            container_id=container_uuid
        ))

        self.assertEqual(200, resp.status_int)
        self.assertEqual(container_name, resp.json.get('name', ''))
        self.assertEqual(container_type, resp.json.get('type', ''))

    def test_should_delete_container(self):
        resp, container_uuid = create_container(
            self.app,
            name='test container name',
            container_type='generic'
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp = self.app.delete('/containers/{container_id}/'.format(
            container_id=container_uuid
        ))

        self.assertEqual(204, resp.status_int)
        self.assertRaises(exception.NotFound, containers_repo.get,
                          container_uuid, self.project_id)

    def test_should_throw_exception_for_get_when_container_not_found(self):
        resp = self.app.get(
            '/containers/{0}/'.format(utils.generate_test_valid_uuid()),
            expect_errors=True)
        self.assertEqual(404, resp.status_int)

    def test_should_throw_exception_for_get_when_invalid_container_id(self):
        resp = self.app.get('/containers/bad_id/', expect_errors=True)
        self.assertEqual(404, resp.status_int)

    def test_should_throw_exception_for_delete_when_container_not_found(self):
        resp = self.app.delete('/containers/bad_id/', expect_errors=True)
        self.assertEqual(404, resp.status_int)
        # Error response should have json content type
        self.assertEqual("application/json", resp.content_type)


class WhenAddingOrRemovingContainerSecretsUsingContainersSecretsResource(
        utils.BarbicanAPIBaseTestCase,
        SuccessfulContainerCreateMixin):

    def test_should_add_container_secret(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            name=secret_name
        )

        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(1, len(container.container_secrets))

    def test_should_add_container_secret_with_trailing_slash(self):
        resp, container_id = create_container(
            self.app,
            name='test container name',
            container_type='generic',
        )
        self._assert_successful_container_create(resp, container_id)

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        request = {
            'name': secret_name,
            'secret_ref': resp.json.get('secret_ref')
        }
        resp = self.app.post_json(
            '/containers/{container_id}/secrets/'.format(
                container_id=container_id
            ),
            request,
            expect_errors=False,
            headers=None
        )
        self.assertEqual(201, resp.status_int)

    def test_should_add_container_secret_without_name(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(1, len(container.container_secrets))

    def test_should_add_container_secret_with_different_name(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            name=secret_name
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(1, len(container.container_secrets))

        secret_name = 'test secret 2'
        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            name=secret_name
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(2, len(container.container_secrets))

    def test_should_not_add_when_secret_not_found(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

        secret_ref = '/secrets/bad_id'
        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            expect_errors=True
        )

        self.assertEqual(404, resp.status_int)

    def test_should_not_add_container_secret_with_invalid_name(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        container_secret_name = "x" * 256
        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            name=container_secret_name,
            expect_errors=True
        )

        self.assertEqual(400, resp.status_int)

    def test_should_not_add_container_secret_with_invalid_secret_ref(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

        secret_ref = ""
        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            expect_errors=True
        )

        self.assertEqual(400, resp.status_int)

    def test_should_add_different_secret_refs_with_duplicate_name(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        first_secret_ref = resp.json.get('secret_ref')

        secret_name = 'test secret 2'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        second_secret_ref = resp.json.get('secret_ref')

        container_secret_name = 'test container secret name'
        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=first_secret_ref,
            name=container_secret_name
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(1, len(container.container_secrets))

        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=second_secret_ref,
            name=container_secret_name
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(2, len(container.container_secrets))

    def test_should_not_allow_add_on_rsa_container(self):
        container_name = 'test container name'
        container_type = 'rsa'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            name=secret_name,
            expect_errors=True
        )

        self.assertEqual(400, resp.status_int)

    def test_should_not_allow_add_on_certificate_container(self):
        container_name = 'test container name'
        container_type = 'certificate'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            name=secret_name,
            expect_errors=True
        )

        self.assertEqual(400, resp.status_int)

    def test_should_not_allow_add_secret_when_exists_in_container(self):
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            name=secret_name
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp, updated_container_uuid = create_container_secret(
            self.app,
            container_id=container_uuid,
            secret_ref=secret_ref,
            name=secret_name,
            expect_errors=True
        )

        self.assertEqual(409, resp.status_int)

    def test_should_delete_existing_container_secret(self):
        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        container_name = 'test container name'
        container_type = 'generic'
        secret_refs = [
            {
                'name': secret_name,
                'secret_ref': secret_ref
            }
        ]
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type,
            secret_refs=secret_refs
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(1, len(container.container_secrets))

        resp = delete_container_secret(self.app, container_uuid, secret_ref,
                                       secret_name)
        self.assertEqual(204, resp.status_int)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

    def test_should_delete_container_secret_without_name(self):
        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        container_name = 'test container name'
        container_type = 'generic'
        secret_refs = [
            {
                'secret_ref': secret_ref
            }
        ]
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type,
            secret_refs=secret_refs
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(1, len(container.container_secrets))

        resp = delete_container_secret(self.app, container_uuid, secret_ref)
        self.assertEqual(204, resp.status_int)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(0, len(container.container_secrets))

    def test_should_not_delete_container_secret_with_incorrect_name(self):
        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        container_name = 'test container name'
        container_type = 'generic'
        secret_refs = [
            {
                'name': secret_name,
                'secret_ref': secret_ref
            }
        ]
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type,
            secret_refs=secret_refs
        )
        self._assert_successful_container_create(resp, container_uuid)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(1, len(container.container_secrets))

        incorrect_name = 'test incorrect name'
        resp = delete_container_secret(self.app, container_uuid, secret_ref,
                                       incorrect_name, expect_errors=True)
        self.assertEqual(404, resp.status_int)

        container = containers_repo.get(container_uuid, self.project_id)
        self.assertEqual(1, len(container.container_secrets))

    def test_should_delete_only_when_secret_exists(self):
        secret_ref = '/secrets/bad_id'
        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp = delete_container_secret(self.app, container_uuid, secret_ref,
                                       expect_errors=True)

        self.assertEqual(404, resp.status_int)

    def test_should_delete_only_when_secret_exists_in_container(self):
        secret_name = 'test secret 1'
        resp, _ = secret_helper.create_secret(
            self.app,
            name=secret_name
        )
        self.assertEqual(201, resp.status_int)
        secret_ref = resp.json.get('secret_ref')

        container_name = 'test container name'
        container_type = 'generic'
        resp, container_uuid = create_container(
            self.app,
            name=container_name,
            container_type=container_type
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp = delete_container_secret(self.app, container_uuid, secret_ref,
                                       secret_name, expect_errors=True)

        self.assertEqual(404, resp.status_int)


class WhenPerformingUnallowedOperationsOnContainers(
        utils.BarbicanAPIBaseTestCase,
        SuccessfulContainerCreateMixin):

    container_req = [
        {
            'name': 'test container name',
            'type': 'generic',
            'secret_refs': []
        }
    ]

    secret_req = {
        'name': 'test secret name',
        'secret_ref': 'https://localhost/v1/secrets/1-2-3-4'
    }

    def test_should_not_allow_put_on_containers(self):
        resp = self.app.put_json(
            '/containers/',
            self.container_req,
            expect_errors=True
        )
        self.assertEqual(405, resp.status_int)

    def test_should_not_allow_post_on_container_by_id(self):
        resp, container_uuid = create_container(
            self.app,
            name='test container name',
            container_type='generic'
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp = self.app.post_json(
            '/containers/{container_id}/'.format(container_id=container_uuid),
            self.container_req,
            expect_errors=True
        )
        self.assertEqual(405, resp.status_int)

    def test_should_not_allow_put_on_container_by_id(self):
        resp, container_uuid = create_container(
            self.app,
            name='test container name',
            container_type='generic'
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp = self.app.put_json(
            '/containers/{container_id}/'.format(container_id=container_uuid),
            self.container_req,
            expect_errors=True
        )
        self.assertEqual(405, resp.status_int)

    def test_should_not_allow_get_on_container_secrets(self):
        resp, container_uuid = create_container(
            self.app,
            name='test container name',
            container_type='generic'
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp = self.app.get(
            '/containers/{container_id}/secrets'.format(
                container_id=container_uuid),
            expect_errors=True
        )
        self.assertEqual(405, resp.status_int)

    def test_should_not_allow_put_on_container_secrets(self):
        resp, container_uuid = create_container(
            self.app,
            name='test container name',
            container_type='generic'
        )
        self._assert_successful_container_create(resp, container_uuid)

        resp = self.app.put_json(
            '/containers/{container_id}/secrets'.format(
                container_id=container_uuid),
            self.secret_req,
            expect_errors=True
        )
        self.assertEqual(405, resp.status_int)


# ----------------------- Helper Functions ---------------------------
def create_container(app, name=None, container_type=None, secret_refs=None,
                     expect_errors=False, headers=None):
    request = {
        'name': name,
        'type': container_type,
        'secret_refs': secret_refs if secret_refs else []
    }
    cleaned_request = {key: val for key, val in request.items()
                       if val is not None}

    resp = app.post_json(
        '/containers/',
        cleaned_request,
        expect_errors=expect_errors,
        headers=headers
    )

    created_uuid = None
    if resp.status_int == 201:
        container_ref = resp.json.get('container_ref', '')
        _, created_uuid = os.path.split(container_ref)

    return resp, created_uuid


def create_container_secret(app, container_id=None, secret_ref=None, name=None,
                            expect_errors=False, headers=None):
    request = {
        'name': name,
        'secret_ref': secret_ref
    }
    cleaned_request = {key: val for key, val in request.items()
                       if val is not None}

    resp = app.post_json(
        '/containers/{container_id}/secrets'.format(container_id=container_id),
        cleaned_request,
        expect_errors=expect_errors,
        headers=headers
    )

    updated_uuid = None
    if resp.status_int == 201:
        container_ref = resp.json.get('container_ref', '')
        _, updated_uuid = os.path.split(container_ref)

    return resp, updated_uuid


def delete_container_secret(app, container_id=None, secret_ref=None, name=None,
                            expect_errors=False, headers=None):

    request = {
        'name': name,
        'secret_ref': secret_ref
    }
    cleaned_request = {key: val for key, val in request.items()
                       if val is not None}

    resp = app.delete_json(
        '/containers/{container_id}/secrets'.format(container_id=container_id),
        cleaned_request,
        expect_errors=expect_errors,
        headers=headers
    )

    return resp



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_policy import policy


rules = [
    policy.RuleDefault(
        name='admin',
        check_str='role:admin'),
    policy.RuleDefault(
        name='observer',
        check_str='role:observer'),
    policy.RuleDefault(
        name='creator',
        check_str='role:creator'),
    policy.RuleDefault(
        name='audit',
        check_str='role:audit'),
    policy.RuleDefault(
        name='service_admin',
        check_str='role:key-manager:service-admin'),
    policy.RuleDefault(
        name='admin_or_creator',
        check_str='rule:admin or rule:creator'),
    policy.RuleDefault(
        name='all_but_audit',
        check_str='rule:admin or rule:observer or rule:creator'),
    policy.RuleDefault(
        name='all_users',
        check_str='rule:admin or rule:observer or rule:creator or ' +
                  'rule:audit or rule:service_admin'),
    policy.RuleDefault(
        name='secret_project_match',
        check_str='project_id:%(target.secret.project_id)s'),
    policy.RuleDefault(
        name='secret_acl_read',
        check_str="'read':%(target.secret.read)s"),
    policy.RuleDefault(
        name='secret_private_read',
        check_str="'False':%(target.secret.read_project_access)s"),
    policy.RuleDefault(
        name='secret_creator_user',
        check_str="user_id:%(target.secret.creator_id)s"),
    policy.RuleDefault(
        name='container_project_match',
        check_str="project_id:%(target.container.project_id)s"),
    policy.RuleDefault(
        name='container_acl_read',
        check_str="'read':%(target.container.read)s"),
    policy.RuleDefault(
        name='container_private_read',
        check_str="'False':%(target.container.read_project_access)s"),
    policy.RuleDefault(
        name='container_creator_user',
        check_str="user_id:%(target.container.creator_id)s"),
    policy.RuleDefault(
        name='secret_non_private_read',
        check_str="rule:all_users and rule:secret_project_match and not " +
                  "rule:secret_private_read"),
    policy.RuleDefault(
        name='secret_decrypt_non_private_read',
        check_str="rule:all_but_audit and rule:secret_project_match and not " +
                  "rule:secret_private_read"),
    policy.RuleDefault(
        name='container_non_private_read',
        check_str="rule:all_users and rule:container_project_match and not " +
                  "rule:container_private_read"),
    policy.RuleDefault(
        name='secret_project_admin',
        check_str="rule:admin and rule:secret_project_match"),
    policy.RuleDefault(
        name='secret_project_creator',
        check_str="rule:creator and rule:secret_project_match and " +
                  "rule:secret_creator_user"),
    policy.RuleDefault(
        name='container_project_admin',
        check_str="rule:admin and rule:container_project_match"),
    policy.RuleDefault(
        name='container_project_creator',
        check_str="rule:creator and rule:container_project_match and " +
                  "rule:container_creator_user"),
]


def list_rules():
    return rules



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_policy import policy


rules = [
    policy.DocumentedRuleDefault(
        name='quotas:get',
        check_str='rule:all_users',
        scope_types=[],
        description='List quotas for the project the user belongs to.',
        operations=[
            {
                'path': '/v1/quotas',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name='project_quotas:get',
        check_str='rule:service_admin',
        scope_types=[],
        description='List quotas for the specified project.',
        operations=[
            {
                'path': '/v1/project-quotas',
                'method': 'GET'
            },
            {
                'path': '/v1/project-quotas/{uuid}',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name='project_quotas:put',
        check_str='rule:service_admin',
        scope_types=[],
        description='Create or update the configured project quotas for '
                    'the project with the specified UUID.',
        operations=[
            {
                'path': '/v1/project-quotas/{uuid}',
                'method': 'PUT'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name='project_quotas:delete',
        check_str='rule:service_admin',
        scope_types=[],
        description='Delete the project quotas configuration for the '
                    'project with the requested UUID.',
        operations=[
            {
                'path': '/v1/quotas}',
                'method': 'DELETE'
            }
        ]
    ),
]


def list_rules():
    return rules



"""
Copyright 2014-2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from functionaltests.api.v1.behaviors import base_behaviors
from functionaltests.api.v1.models import consumer_model


class ConsumerBehaviors(base_behaviors.BaseBehaviors):

    def create_consumer(self, model, container_ref, extra_headers=None,
                        user_name=None, admin=None, use_auth=True):
        """Register a consumer to a container.

        :param model: The metadata for the consumer
        :param container_ref: Full reference to a container
        :param extra_headers: Any additional headers to pass to the request
        :param user_name: The user name used to create the consumer
        :param admin: The user with permission to delete the consumer
        :param use_auth: Boolean for whether to send authentication headers

        :return: A tuple containing the response from the create
        and the href to the newly registered consumer
        """

        url = '{0}/consumers'.format(container_ref)

        resp = self.client.post(url, request_model=model,
                                extra_headers=extra_headers,
                                user_name=user_name, use_auth=use_auth)

        if resp.status_code == 401 and not use_auth:
            return resp, None

        if resp.status_code == 200:
            if admin is None:
                admin = user_name
            self.created_entities.append((container_ref, model, admin))

        returned_data = self.get_json(resp)
        consumer_data = returned_data.get('consumers')

        return resp, consumer_data

    def get_consumers(self, container_ref, limit=10, offset=0,
                      extra_headers=None,
                      user_name=None, use_auth=True):
        """Gets a list of consumers on a container.

        :param container_ref: Full reference to a container
        :param limit: limits number of returned consumers
        :param offset: represents how many records to skip before retrieving
            the list
        :param extra_headers: Any additional headers to pass to the request
        :param user_name: The user name used to get the consumer
        :param use_auth: Boolean for whether to send authentication headers

        :return: The response from the get and refs to the next/previous list
            of consumers
        """

        url = '{0}/consumers'.format(container_ref)

        params = {'limit': limit, 'offset': offset}
        resp = self.client.get(url, params=params, extra_headers=extra_headers,
                               user_name=user_name, use_auth=use_auth)

        if resp.status_code == 401 and not use_auth:
            return resp, None, None, None

        consumer_list = self.get_json(resp)

        consumers, next_ref, prev_ref = self.client.get_list_of_models(
            consumer_list, consumer_model.ConsumerModel)

        return resp, consumers, next_ref, prev_ref

    def delete_consumer(self, model, container_ref, extra_headers=None,
                        user_name=None, use_auth=True):
        """Deletes a consumer from a container.

        :param model: The metadata for the consumer
        :param container_ref: Full reference to a container
        :param extra_headers: Any additional headers to pass to the request
        :param user_name: The user name used to delete the consumer
        :param use_auth: Boolean for whether to send authentication headers

        :return: The response from the delete
        """
        url = '{0}/consumers'.format(container_ref)

        resp = self.client.delete(url, request_model=model,
                                  extra_headers=extra_headers,
                                  user_name=user_name,
                                  use_auth=use_auth)

        if resp.status_code == 401 and not use_auth:
            return resp, None

        if resp.status_code != 200:
            return resp, None

        returned_data = self.get_json(resp)
        consumer_data = returned_data['consumers']

        return resp, consumer_data

    def delete_all_created_consumers(self):
        """Delete all of the consumers that we have created."""
        entities = list(self.created_entities)
        for (container_ref, model, admin) in entities:
            self.delete_consumer(model, container_ref, user_name=admin)



# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from barbican.common import config
from barbican import i18n as u

import barbican.plugin.interface.certificate_manager as cm

CONF = config.new_config()

dogtag_plugin_group = cfg.OptGroup(name='dogtag_plugin',
                                   title="Dogtag Plugin Options")
dogtag_plugin_opts = [
    cfg.StrOpt('pem_path',
               default='/etc/barbican/kra_admin_cert.pem',
               help=u._('Path to PEM file for authentication')),
    cfg.StrOpt('dogtag_host',
               default="localhost",
               help=u._('Hostname for the Dogtag instance')),
    cfg.PortOpt('dogtag_port',
                default=8443,
                help=u._('Port for the Dogtag instance')),
    cfg.StrOpt('nss_db_path',
               default='/etc/barbican/alias',
               help=u._('Path to the NSS certificate database')),
    cfg.StrOpt('nss_password',
               help=u._('Password for the NSS certificate databases'),
               secret=True),
    cfg.StrOpt('simple_cmc_profile',
               default='caOtherCert',
               help=u._('Profile for simple CMC requests')),
    cfg.StrOpt('auto_approved_profiles',
               default="caServerCert",
               help=u._('List of automatically approved enrollment profiles')),
    cfg.IntOpt('ca_expiration_time',
               default=cm.CA_INFO_DEFAULT_EXPIRATION_DAYS,
               help=u._('Time in days for CA entries to expire')),
    cfg.StrOpt('plugin_working_dir',
               default='/etc/barbican/dogtag',
               help=u._('Working directory for Dogtag plugin')),
    cfg.StrOpt('plugin_name',
               help=u._('User friendly plugin name'),
               default='Dogtag KRA'),
    cfg.IntOpt('retries',
               help=u._('Retries when storing or generating secrets'),
               default=3)
]

CONF.register_group(dogtag_plugin_group)
CONF.register_opts(dogtag_plugin_opts, group=dogtag_plugin_group)
config.parse_args(CONF)


def list_opts():
    yield dogtag_plugin_group, dogtag_plugin_opts



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Common utilities for Barbican.
"""
import collections
import importlib
import mimetypes
import uuid

from oslo_log import log
from oslo_utils import uuidutils
import pecan
import re
import six
from six.moves.urllib import parse

from barbican.common import config
from barbican import i18n as u


CONF = config.CONF


# Current API version
API_VERSION = 'v1'

# Added here to remove cyclic dependency.
# In barbican.model.models module SecretType.OPAQUE was imported from
# barbican.plugin.interface.secret_store which introduces a cyclic dependency
# if `secret_store` plugin needs to use db model classes. So moving shared
# value to another common python module which is already imported in both.
SECRET_TYPE_OPAQUE = "opaque"  # nosec


def _do_allow_certain_content_types(func, content_types_list=[]):
    # Allows you to bypass pecan's content-type restrictions
    cfg = pecan.util._cfg(func)
    cfg.setdefault('content_types', {})
    cfg['content_types'].update((value, '')
                                for value in content_types_list)
    return func


def allow_certain_content_types(*content_types_list):
    def _wrapper(func):
        return _do_allow_certain_content_types(func, content_types_list)
    return _wrapper


def allow_all_content_types(f):
    return _do_allow_certain_content_types(f, mimetypes.types_map.values())


def get_base_url_from_request():
    """Derive base url from wsgi request if CONF.host_href is not set

    Use host.href as base URL if its set in barbican.conf.
    If its not set, then derives value from wsgi request. WSGI request uses
    HOST header or HTTP_X_FORWARDED_FOR header (in case of proxy) for host +
    port part of its url. Proxies can also set HTTP_X_FORWARDED_PROTO header
    for indicating http vs https.

    Some of unit tests does not have pecan context that's why using request
    attr check on pecan instance.
    """
    if not CONF.host_href and hasattr(pecan.request, 'application_url'):
        p_url = parse.urlsplit(pecan.request.application_url)
        # Pecan does not handle X_FORWARDED_PROTO yet, so we need to
        # handle it ourselves. see lp#1445290
        scheme = pecan.request.environ.get('HTTP_X_FORWARDED_PROTO', 'http')
        # Pecan does not handle url reconstruction according to
        # https://www.python.org/dev/peps/pep-0333/#url-reconstruction
        netloc = pecan.request.environ.get('HTTP_HOST', p_url.netloc)
        # FIXME: implement SERVER_NAME lookup if HTTP_HOST is not set
        if p_url.path:
            # Remove the version from the path to extract the base path
            base_path = re.sub('/v[0-9\.]+$', '', p_url.path)
            base_url = '%s://%s%s' % (scheme, netloc, base_path)
        else:
            base_url = '%s://%s' % (scheme, netloc)
        return base_url
    else:  # when host_href is set or flow is not within wsgi request context
        return CONF.host_href


def hostname_for_refs(resource=None):
    """Return the HATEOAS-style return URI reference for this service."""
    base_url = get_base_url_from_request()
    ref = ['{base}/{version}'.format(base=base_url, version=API_VERSION)]
    if resource:
        ref.append('/' + resource)
    return ''.join(ref)


# Return a logger instance.
#   Note: Centralize access to the logger to avoid the dreaded
#   'ArgsAlreadyParsedError: arguments already parsed: cannot
#   register CLI option'
#   error.
def getLogger(name):
    return log.getLogger(name)


def get_accepted_encodings(req):
    """Returns a list of client acceptable encodings sorted by q value.

    For details see: http://tools.ietf.org/html/rfc2616#section-14.3

    :param req: request object
    :returns: list of client acceptable encodings sorted by q value.
    """
    header = req.get_header('Accept-Encoding')

    return get_accepted_encodings_direct(header)


def get_accepted_encodings_direct(content_encoding_header):
    """Returns a list of client acceptable encodings sorted by q value.

    For details see: http://tools.ietf.org/html/rfc2616#section-14.3

    :param req: request object
    :returns: list of client acceptable encodings sorted by q value.
    """
    if content_encoding_header is None:
        return None

    Encoding = collections.namedtuple('Encoding', ['coding', 'quality'])

    encodings = list()
    for enc in content_encoding_header.split(','):
        if ';' in enc:
            coding, qvalue = enc.split(';')
            try:
                qvalue = qvalue.split('=')[1]
                quality = float(qvalue.strip())
            except ValueError:
                # can't convert quality to float
                return None
            if quality > 1.0 or quality < 0.0:
                # quality is outside valid range
                return None
            if quality > 0.0:
                encodings.append(Encoding(coding.strip(), quality))
        else:
            encodings.append(Encoding(enc.strip(), 1))

    # Sort the encodings by quality
    encodings = sorted(encodings, key=lambda e: e.quality, reverse=True)

    return [encoding.coding for encoding in encodings]


def generate_fullname_for(instance):
    """Produce a fully qualified class name for the specified instance.

    :param instance: The instance to generate information from.
    :return: A string providing the package.module information for the
    instance.
    :raises: ValueError if the given instance is null
    """
    if not instance:
        raise ValueError(u._("Cannot generate a fullname for a null instance"))

    module = type(instance).__module__
    class_name = type(instance).__name__

    if module is None or module == six.moves.builtins.__name__:
        return class_name
    return "{module}.{class_name}".format(module=module, class_name=class_name)


def get_class_for(module_name, class_name):
    """Create a Python class from its text-specified components."""
    # Load the module via name, raising ImportError if module cannot be
    # loaded.
    python_module = importlib.import_module(module_name)

    # Load and return the resolved Python class, raising AttributeError if
    # class cannot be found.
    return getattr(python_module, class_name)


def generate_uuid():
    return uuidutils.generate_uuid()


def is_multiple_backends_enabled():
    try:
        secretstore_conf = config.get_module_config('secretstore')
    except KeyError:
        # Ensure module is initialized
        from barbican.plugin.interface import secret_store  # nopep8
        secretstore_conf = config.get_module_config('secretstore')
    return secretstore_conf.secretstore.enable_multiple_secret_stores


def validate_id_is_uuid(input_id, version=4):
    """Validates provided id is uuid4 format value.

    Returns true when provided id is a valid version 4 uuid otherwise
    returns False.
    This validation is to be used only for ids which are generated by barbican
    (e.g. not for keystone project_id)
    """

    try:
        value = uuid.UUID(input_id, version=version)
    except Exception:
        return False
    return str(value) == input_id



#!/usr/bin/env python

# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Barbican worker server, running a periodic retry/scheduler process.
"""

import eventlet
import os
import sys

# Oslo messaging RPC server uses eventlet.
eventlet.monkey_patch()

# 'Borrowed' from the Glance project:
# If ../barbican/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                   os.pardir,
                                   os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'barbican', '__init__.py')):
    sys.path.insert(0, possible_topdir)


from barbican.common import config
from barbican import queue
from barbican.queue import retry_scheduler
from barbican import version

from oslo_log import log
from oslo_service import service


def fail(returncode, e):
    sys.stderr.write("ERROR: {0}\n".format(e))
    sys.exit(returncode)


def main():
    try:
        CONF = config.CONF
        CONF(sys.argv[1:], project='barbican',
             version=version.version_info.version_string)

        # Import and configure logging.
        log.setup(CONF, 'barbican-retry-scheduler')
        LOG = log.getLogger(__name__)
        LOG.debug("Booting up Barbican worker retry/scheduler node...")

        # Queuing initialization (as a client only).
        queue.init(CONF, is_server_side=False)

        service.launch(
            CONF,
            retry_scheduler.PeriodicServer(),
            restart_method='mutate'
        ).wait()
    except RuntimeError as e:
        fail(1, e)


if __name__ == '__main__':
    main()



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import unittest

from barbican.common import exception
from barbican.model import models
from barbican.plugin.interface import secret_store
from barbican.tests import utils


class WhenCreatingNewSecret(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewSecret, self).setUp()
        self.parsed_secret = {'name': 'name',
                              'secret_type': secret_store.SecretType.OPAQUE,
                              'algorithm': 'algorithm',
                              'bit_length': 512,
                              'mode': 'mode',
                              'plain_text': 'not-encrypted',
                              'creator_id': 'creator12345'}

        self.parsed_order = {'secret': self.parsed_secret}

    def test_new_secret_is_created_from_dict(self):
        date_time = datetime.datetime.utcnow().isoformat()
        self.parsed_secret['expiration'] = date_time
        secret = models.Secret(self.parsed_secret)
        self.assertEqual(self.parsed_secret['name'], secret.name)
        self.assertEqual(self.parsed_secret['secret_type'], secret.secret_type)
        self.assertEqual(self.parsed_secret['algorithm'], secret.algorithm)
        self.assertEqual(self.parsed_secret['bit_length'], secret.bit_length)
        self.assertEqual(self.parsed_secret['mode'], secret.mode)
        self.assertIsInstance(secret.expiration, datetime.datetime)
        self.assertEqual(self.parsed_secret['creator_id'], secret.creator_id)
        self.assertEqual(secret.created_at, secret.updated_at)

        fields = secret.to_dict_fields()
        self.assertEqual(self.parsed_secret['secret_type'],
                         fields['secret_type'])
        self.assertEqual(self.parsed_secret['algorithm'], fields['algorithm'])
        self.assertEqual(self.parsed_secret['creator_id'],
                         fields['creator_id'])

    def test_new_secret_is_created_with_default_secret_type(self):
        secret_spec = dict(self.parsed_secret)
        date_time = datetime.datetime.utcnow().isoformat()
        secret_spec['expiration'] = date_time
        del secret_spec['secret_type']
        secret = models.Secret(secret_spec)
        self.assertEqual(self.parsed_secret['secret_type'], secret.secret_type)


class WhenCreatingNewSecretMetadata(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewSecretMetadata, self).setUp()
        self.key = 'dog'
        self.value = 'poodle'

        self.metadata = {
            'key': self.key,
            'value': self.value
        }

    def test_new_secret_metadata_is_created_from_dict(self):
        secret_meta = models.SecretUserMetadatum(self.key, self.value)

        self.assertEqual(self.key, secret_meta.key)
        self.assertEqual(self.value, secret_meta.value)

        fields = secret_meta.to_dict_fields()
        self.assertEqual(self.metadata['key'],
                         fields['key'])
        self.assertEqual(self.metadata['value'],
                         fields['value'])

    def test_should_raise_exception_metadata_with_no_key(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretUserMetadatum,
                          None,
                          self.value)

    def test_should_raise_exception_metadata_with_no_value(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretUserMetadatum,
                          self.key,
                          None)


class WhenCreatingNewOrder(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewOrder, self).setUp()
        self.parsed_order = {
            'type': 'certificate',
            'meta': {
                'email': 'email@email.com'
            },
            'sub_status': 'Pending',
            'sub_status_message': 'Waiting for instructions...',
            'creator_id': 'creator12345'
        }

    def test_new_order_is_created(self):
        order = models.Order(self.parsed_order)

        self.assertEqual(self.parsed_order['type'], order.type)
        self.assertEqual(self.parsed_order['meta'], order.meta)
        self.assertEqual(self.parsed_order['sub_status'], order.sub_status)
        self.assertEqual(self.parsed_order['creator_id'], order.creator_id)
        self.assertEqual(
            self.parsed_order['sub_status_message'],
            order.sub_status_message
        )
        fields = order.to_dict_fields()
        self.assertEqual(self.parsed_order['sub_status'], fields['sub_status'])
        self.assertEqual(self.parsed_order['type'], fields['type'])
        self.assertEqual(self.parsed_order['creator_id'],
                         fields['creator_id'])


class WhenCreatingNewContainer(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewContainer, self).setUp()
        self.parsed_container = {'name': 'name',
                                 'type': 'generic',
                                 'secret_refs': [
                                     {'name': 'test secret 1',
                                      'secret_ref': '123'},
                                     {'name': 'test secret 2',
                                      'secret_ref': '123'},
                                     {'name': 'test secret 3',
                                      'secret_ref': '123'}
                                 ],
                                 'creator_id': 'creator123456'}

    def test_new_container_is_created_from_dict(self):
        container = models.Container(self.parsed_container)
        self.assertEqual(self.parsed_container['name'], container.name)
        self.assertEqual(self.parsed_container['type'], container.type)
        self.assertEqual(self.parsed_container['creator_id'],
                         container.creator_id)
        self.assertEqual(len(self.parsed_container['secret_refs']),
                         len(container.container_secrets))

        self.assertEqual(self.parsed_container['secret_refs'][0]['name'],
                         container.container_secrets[0].name)
        self.assertEqual(self.parsed_container['secret_refs'][0]['secret_ref'],
                         container.container_secrets[0].secret_id)

        self.assertEqual(self.parsed_container['secret_refs'][1]['name'],
                         container.container_secrets[1].name)
        self.assertEqual(self.parsed_container['secret_refs'][1]['secret_ref'],
                         container.container_secrets[1].secret_id)

        self.assertEqual(self.parsed_container['secret_refs'][2]['name'],
                         container.container_secrets[2].name)
        self.assertEqual(self.parsed_container['secret_refs'][2]['secret_ref'],
                         container.container_secrets[2].secret_id)
        fields = container.to_dict_fields()
        self.assertEqual(self.parsed_container['name'], fields['name'])
        self.assertEqual(self.parsed_container['type'], fields['type'])
        self.assertEqual(self.parsed_container['creator_id'],
                         fields['creator_id'])

    def test_new_certificate_container_is_created_from_dict(self):
        self.parsed_container['type'] = 'certificate'
        container = models.Container(self.parsed_container)
        self.assertEqual(self.parsed_container['name'], container.name)
        self.assertEqual(self.parsed_container['type'], container.type)
        self.assertEqual(self.parsed_container['creator_id'],
                         container.creator_id)
        self.assertEqual(len(self.parsed_container['secret_refs']),
                         len(container.container_secrets))

        self.assertEqual(self.parsed_container['secret_refs'][0]['name'],
                         container.container_secrets[0].name)
        self.assertEqual(self.parsed_container['secret_refs'][0]['secret_ref'],
                         container.container_secrets[0].secret_id)

        self.assertEqual(self.parsed_container['secret_refs'][1]['name'],
                         container.container_secrets[1].name,)
        self.assertEqual(self.parsed_container['secret_refs'][1]['secret_ref'],
                         container.container_secrets[1].secret_id)

        self.assertEqual(self.parsed_container['secret_refs'][2]['name'],
                         container.container_secrets[2].name)
        self.assertEqual(self.parsed_container['secret_refs'][2]['secret_ref'],
                         container.container_secrets[2].secret_id)

    def test_parse_secret_ref_uri(self):
        self.parsed_container['secret_refs'][0]['secret_ref'] = (
            'http://localhost:9110/123/secrets/123456')
        container = models.Container(self.parsed_container)
        self.assertEqual('123456', container.container_secrets[0].secret_id)

        self.parsed_container['secret_refs'][0]['secret_ref'] = (
            'http://localhost:9110/123/secrets/123456/')
        container = models.Container(self.parsed_container)
        self.assertEqual('123456', container.container_secrets[0].secret_id)


class WhenCreatingNewConsumer(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewConsumer, self).setUp()
        self.parsed_consumer = {'name': 'name',
                                'URL': 'URL'}
        self.project_id = '12345project'
        self.container_id = '12345container'

    def test_new_consumer_is_created_from_dict(self):
        consumer = models.ContainerConsumerMetadatum(self.container_id,
                                                     self.project_id,
                                                     self.parsed_consumer)
        self.assertEqual(self.parsed_consumer['name'], consumer.name)
        self.assertEqual(self.parsed_consumer['URL'], consumer.URL)
        self.assertEqual(models.States.ACTIVE, consumer.status)

    def test_new_consumer_has_correct_hash(self):
        consumer_one = models.ContainerConsumerMetadatum(self.container_id,
                                                         self.project_id,
                                                         self.parsed_consumer)
        consumer_two = models.ContainerConsumerMetadatum(self.container_id,
                                                         self.project_id,
                                                         self.parsed_consumer)
        different_container = '67890container'
        consumer_three = models.ContainerConsumerMetadatum(
            different_container, self.project_id, self.parsed_consumer)
        self.assertEqual(consumer_one.data_hash, consumer_two.data_hash)
        self.assertNotEqual(consumer_one.data_hash, consumer_three.data_hash)


class WhenProcessingJsonBlob(utils.BaseTestCase):
    def setUp(self):
        super(WhenProcessingJsonBlob, self).setUp()
        self.json_blob = models.JsonBlob()

    def test_process_bind_param_w_dict(self):
        res = self.json_blob.process_bind_param({'test': True}, None)
        self.assertEqual('{"test": true}', res)

    def test_process_result_value_w_json_str(self):
        res = self.json_blob.process_result_value('{"test": true}', None)
        self.assertTrue(res.get('test'))


class WhenCreatingOrderRetryTask(utils.BaseTestCase):

    def test_create_new_order_task(self):
        order = models.Order({
            'type': 'certificate',
            'meta': {
                'email': 'email@email.com'
            },
            'sub_status': 'Pending',
            'sub_status_message': 'Waiting for instructions...'
        })
        at = datetime.datetime.utcnow()
        order_retry_task = models.OrderRetryTask()
        order_retry_task.order_id = order.id
        order_retry_task.retry_task = "foobar"
        order_retry_task.retry_at = at
        order_retry_task.retry_args = ["one", "two"]
        order_retry_task.retry_kwargs = {"three": "four"}

        self.assertEqual(order.id, order_retry_task.order_id)
        self.assertEqual("foobar", order_retry_task.retry_task)
        self.assertEqual(at, order_retry_task.retry_at)
        self.assertEqual(
            ["one", "two"],
            order_retry_task.retry_args,
        )
        self.assertEqual(
            {"three": "four"},
            order_retry_task.retry_kwargs,
        )


class WhenCreatingNewCertificateAuthority(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewCertificateAuthority, self).setUp()
        expiration = (datetime.datetime.utcnow() +
                      datetime.timedelta(minutes=10))
        self.parsed_ca = {'plugin_name': 'dogtag_plugin',
                          'plugin_ca_id': 'ca_master',
                          'expiration': expiration.isoformat(),
                          'name': 'Dogtag CA',
                          'description': 'Master CA for Dogtag plugin',
                          'ca_signing_certificate': 'XXXXX',
                          'intermediates': 'YYYYY',
                          'creator_id': 'user12345',
                          'parent_ca_id': '12330-223-22',
                          'project_id': '12345'}

    def test_new_ca_is_created_from_dict(self):
        ca = models.CertificateAuthority(self.parsed_ca)
        self.assertEqual(self.parsed_ca['plugin_name'], ca.plugin_name)
        self.assertEqual(self.parsed_ca['plugin_ca_id'], ca.plugin_ca_id)
        self.assertEqual(self.parsed_ca['name'], ca.ca_meta['name'].value)
        self.assertEqual(self.parsed_ca['description'],
                         ca.ca_meta['description'].value)
        self.assertEqual(self.parsed_ca['ca_signing_certificate'],
                         ca.ca_meta['ca_signing_certificate'].value)
        self.assertEqual(self.parsed_ca['intermediates'],
                         ca.ca_meta['intermediates'].value)
        self.assertIsInstance(ca.expiration, datetime.datetime)
        self.assertEqual(ca.created_at, ca.updated_at)
        self.assertEqual(self.parsed_ca['creator_id'], ca.creator_id)
        self.assertEqual(self.parsed_ca['project_id'], ca.project_id)


class WhenCreatingNewProjectCertificateAuthority(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewProjectCertificateAuthority, self).setUp()
        expiration = (datetime.datetime.utcnow() +
                      datetime.timedelta(minutes=10))
        self.parsed_ca = {'plugin_name': 'dogtag_plugin',
                          'plugin_ca_id': 'ca_master',
                          'expiration': expiration.isoformat(),
                          'name': 'Dogtag CA',
                          'description': 'Master CA for Dogtag plugin',
                          'ca_signing_certificate': 'XXXXX',
                          'intermediates': 'YYYYY'}

    def test_create_new_project_ca(self):
        ca = models.CertificateAuthority(self.parsed_ca)
        ca.id = '67890'
        project = models.Project()
        project.id = '12345'
        project_ca = models.ProjectCertificateAuthority(project.id, ca.id)

        self.assertEqual(ca.id, project_ca.ca_id)
        self.assertEqual(project.id, project_ca.project_id)


class WhenCreatingNewPreferredCertificateAuthority(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewPreferredCertificateAuthority, self).setUp()
        expiration = (datetime.datetime.utcnow() +
                      datetime.timedelta(minutes=10))
        self.parsed_ca = {'plugin_name': 'dogtag_plugin',
                          'plugin_ca_id': 'ca_master',
                          'expiration': expiration.isoformat(),
                          'name': 'Dogtag CA',
                          'description': 'Master CA for Dogtag plugin',
                          'ca_signing_certificate': 'XXXXX',
                          'intermediates': 'YYYYY'}

    def test_create_new_preferred_ca(self):
        ca = models.CertificateAuthority(self.parsed_ca)
        ca.id = '67890'
        project = models.Project()
        project.id = '12345'
        preferred_ca = models.PreferredCertificateAuthority(project.id, ca.id)

        self.assertEqual(ca.id, preferred_ca.ca_id)
        self.assertEqual(project.id, preferred_ca.project_id)


class WhenCreatingNewSecretACL(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewSecretACL, self).setUp()
        self.secret_id = 'secret123456'
        self.user_ids = ['user12345', 'user67890']
        self.operation = 'read'
        self.project_access = True

    def test_new_secretacl_for_given_all_input(self):
        acl = models.SecretACL(self.secret_id, self.operation,
                               self.project_access, self.user_ids)
        self.assertEqual(self.secret_id, acl.secret_id)
        self.assertEqual(self.operation, acl.operation)
        self.assertEqual(self.project_access, acl.project_access)
        self.assertTrue(all(acl_user.user_id in self.user_ids for acl_user
                            in acl.acl_users))

    def test_new_secretacl_check_to_dict_fields(self):
        acl = models.SecretACL(self.secret_id, self.operation,
                               self.project_access, self.user_ids)
        self.assertEqual(self.secret_id, acl.to_dict_fields()['secret_id'])
        self.assertEqual(self.operation, acl.to_dict_fields()['operation'])
        self.assertEqual(self.project_access,
                         acl.to_dict_fields()['project_access'])
        self.assertTrue(all(user_id in self.user_ids for user_id in
                            acl.to_dict_fields()['users']))
        self.assertIsNone(acl.to_dict_fields()['acl_id'])

    def test_new_secretacl_for_bare_minimum_input(self):
        acl = models.SecretACL(self.secret_id, self.operation,
                               None, None)
        self.assertEqual(self.secret_id, acl.secret_id)
        self.assertEqual(0, len(acl.acl_users))
        self.assertEqual(self.operation, acl.operation)
        self.assertIsNone(acl.project_access)

    def test_new_secretacl_with_duplicate_userids_input(self):
        user_ids = list(self.user_ids)
        user_ids *= 2  # duplicate ids
        acl = models.SecretACL(self.secret_id, self.operation,
                               None, user_ids=user_ids)
        self.assertEqual(self.secret_id, acl.secret_id)
        self.assertEqual(self.operation, acl.operation)
        self.assertIsNone(acl.project_access)
        self.assertEqual(2, len(acl.acl_users))

    def test_should_throw_exception_missing_secret_id(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretACL, None, 'read',
                          ['user246'], None)

    def test_should_throw_exception_missing_operation(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretACL, self.secret_id, None,
                          None, ['user246'])

    def test_new_secretacl_expect_user_ids_as_list(self):
        acl = models.SecretACL(self.secret_id, self.operation,
                               None, {'aUser': '12345'})
        self.assertEqual(0, len(acl.acl_users))


class WhenCreatingNewContainerACL(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewContainerACL, self).setUp()
        self.container_id = 'container123456'
        self.user_ids = ['user12345', 'user67890']
        self.operation = 'read'
        self.project_access = True

    def test_new_containeracl_for_given_all_input(self):
        acl = models.ContainerACL(self.container_id, self.operation,
                                  self.project_access, self.user_ids)
        self.assertEqual(self.container_id, acl.container_id)
        self.assertEqual(self.operation, acl.operation)
        self.assertEqual(self.project_access, acl.project_access)
        self.assertTrue(all(acl_user.user_id in self.user_ids for acl_user
                            in acl.acl_users))

    def test_new_containeracl_check_to_dict_fields(self):
        acl = models.ContainerACL(self.container_id, self.operation,
                                  self.project_access, self.user_ids)
        self.assertEqual(self.container_id,
                         acl.to_dict_fields()['container_id'])
        self.assertEqual(self.operation, acl.to_dict_fields()['operation'])
        self.assertEqual(self.project_access,
                         acl.to_dict_fields()['project_access'])
        self.assertTrue(all(user_id in self.user_ids for user_id
                            in acl.to_dict_fields()['users']))
        self.assertIsNone(acl.to_dict_fields()['acl_id'])

    def test_new_containeracl_for_bare_minimum_input(self):
        acl = models.ContainerACL(self.container_id, self.operation,
                                  None, None)
        self.assertEqual(self.container_id, acl.container_id)
        self.assertEqual(0, len(acl.acl_users))
        self.assertEqual(self.operation, acl.operation)
        self.assertIsNone(acl.project_access)

    def test_new_containeracl_with_duplicate_userids_input(self):
        user_ids = list(self.user_ids)
        user_ids *= 2  # duplicate ids
        acl = models.ContainerACL(self.container_id, self.operation,
                                  True, user_ids=user_ids)
        self.assertEqual(self.container_id, acl.container_id)
        self.assertEqual(self.operation, acl.operation)
        self.assertTrue(acl.project_access)
        self.assertEqual(2, len(acl.acl_users))

    def test_should_throw_exception_missing_container_id(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.ContainerACL, None, 'read',
                          None, ['user246'])

    def test_should_throw_exception_missing_operation(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.ContainerACL, self.container_id, None,
                          None, ['user246'])

    def test_new_containeracl_expect_user_ids_as_list(self):
        acl = models.ContainerACL(self.container_id, self.operation,
                                  None, {'aUser': '12345'})
        self.assertEqual(0, len(acl.acl_users))


class WhenCreatingNewSecretACLUser(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewSecretACLUser, self).setUp()
        self.secret_acl_id = 'secret_acl_123456'
        self.user_ids = ['user12345', 'user67890']

    def test_new_secretacl_user_for_given_all_input(self):
        acl_user = models.SecretACLUser(self.secret_acl_id, self.user_ids[0])

        self.assertEqual(self.secret_acl_id, acl_user.acl_id)
        self.assertEqual(self.user_ids[0], acl_user.user_id)
        self.assertEqual(models.States.ACTIVE, acl_user.status)

    def test_new_secretacl_user_check_to_dict_fields(self):
        acl_user = models.SecretACLUser(self.secret_acl_id, self.user_ids[1])

        self.assertEqual(self.secret_acl_id,
                         acl_user.to_dict_fields()['acl_id'])
        self.assertEqual(self.user_ids[1],
                         acl_user.to_dict_fields()['user_id'])
        self.assertEqual(models.States.ACTIVE,
                         acl_user.to_dict_fields()['status'])

    def test_should_throw_exception_missing_user_id(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretACLUser, self.secret_acl_id,
                          None)


class WhenCreatingNewContainerACLUser(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewContainerACLUser, self).setUp()
        self.container_acl_id = 'container_acl_123456'
        self.user_ids = ['user12345', 'user67890']

    def test_new_secretacl_user_for_given_all_input(self):
        acl_user = models.ContainerACLUser(self.container_acl_id,
                                           self.user_ids[0])

        self.assertEqual(self.container_acl_id, acl_user.acl_id)
        self.assertEqual(self.user_ids[0], acl_user.user_id)
        self.assertEqual(models.States.ACTIVE, acl_user.status)

    def test_new_secretacl_user_check_to_dict_fields(self):
        acl_user = models.ContainerACLUser(self.container_acl_id,
                                           self.user_ids[1])

        self.assertEqual(self.container_acl_id,
                         acl_user.to_dict_fields()['acl_id'])
        self.assertEqual(self.user_ids[1],
                         acl_user.to_dict_fields()['user_id'])
        self.assertEqual(models.States.ACTIVE,
                         acl_user.to_dict_fields()['status'])

    def test_should_throw_exception_missing_user_id(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.ContainerACLUser, self.container_acl_id,
                          None)


class WhenCreatingNewProjectQuotas(utils.BaseTestCase):
    def test_create_new_project_quotas(self):
        project = models.Project()
        project.id = '12345'
        project.external_id = '67890'
        parsed_project_quotas = {
            'secrets': 101,
            'orders': 102,
            'containers': 103,
            'consumers': 105,
            'cas': 106}
        project_quotas = models.ProjectQuotas(project.id,
                                              parsed_project_quotas)

        self.assertEqual('12345', project_quotas.project_id)
        self.assertEqual(101, project_quotas.secrets)
        self.assertEqual(102, project_quotas.orders)
        self.assertEqual(103, project_quotas.containers)
        self.assertEqual(105, project_quotas.consumers)
        self.assertEqual(106, project_quotas.cas)

    def test_create_new_project_quotas_with_all_default_quotas(self):
        project = models.Project()
        project.id = '12345'
        project.external_id = '67890'
        project_quotas = models.ProjectQuotas(project.id,
                                              None)

        self.assertEqual('12345', project_quotas.project_id)
        self.assertIsNone(project_quotas.secrets)
        self.assertIsNone(project_quotas.orders)
        self.assertIsNone(project_quotas.containers)
        self.assertIsNone(project_quotas.consumers)
        self.assertIsNone(project_quotas.cas)

    def test_create_new_project_quotas_with_some_default_quotas(self):
        project = models.Project()
        project.id = '12345'
        project.external_id = '67890'
        parsed_project_quotas = {
            'secrets': 101,
            'containers': 103,
            'consumers': 105}
        project_quotas = models.ProjectQuotas(project.id,
                                              parsed_project_quotas)

        self.assertEqual('12345', project_quotas.project_id)
        self.assertEqual(101, project_quotas.secrets)
        self.assertIsNone(project_quotas.orders)
        self.assertEqual(103, project_quotas.containers)
        self.assertEqual(105, project_quotas.consumers)
        self.assertIsNone(project_quotas.cas)

    def test_should_throw_exception_missing_project_id(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.ProjectQuotas, None, None)

    def test_project_quotas_check_to_dict_fields(self):
        project = models.Project()
        project.id = '12345'
        project.external_id = '67890'
        parsed_project_quotas = {
            'secrets': 101,
            'orders': 102,
            'containers': 103,
            'consumers': 105,
            'cas': 106}
        project_quotas = models.ProjectQuotas(project.id,
                                              parsed_project_quotas)
        self.assertEqual(project.id,
                         project_quotas.to_dict_fields()['project_id'])
        self.assertEqual(101,
                         project_quotas.to_dict_fields()['secrets'])
        self.assertEqual(102,
                         project_quotas.to_dict_fields()['orders'])
        self.assertEqual(103,
                         project_quotas.to_dict_fields()['containers'])
        self.assertEqual(105,
                         project_quotas.to_dict_fields()['consumers'])
        self.assertEqual(106,
                         project_quotas.to_dict_fields()['cas'])


class WhenCreatingNewSecretStores(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewSecretStores, self).setUp()

    def test_new_secret_stores_for_all_input(self):
        name = "db backend"
        store_plugin = 'store_crypto'
        crypto_plugin = 'simple_crypto'
        ss = models.SecretStores(name, store_plugin, crypto_plugin,
                                 global_default=True)

        self.assertEqual(store_plugin, ss.store_plugin)
        self.assertEqual(crypto_plugin, ss.crypto_plugin)
        self.assertEqual(name, ss.name)
        self.assertTrue(ss.global_default)
        self.assertEqual(models.States.ACTIVE, ss.status)

    def test_new_secret_stores_required_input_only(self):
        store_plugin = 'store_crypto'
        name = "db backend"
        ss = models.SecretStores(name, store_plugin)

        self.assertEqual(store_plugin, ss.store_plugin)
        self.assertEqual(name, ss.name)
        self.assertIsNone(ss.crypto_plugin)
        self.assertIsNone(ss.global_default)  # False default is not used
        self.assertEqual(models.States.ACTIVE, ss.status)

    def test_should_throw_exception_missing_store_plugin(self):
        name = "db backend"
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretStores, name, None)
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretStores, name, "")

    def test_should_throw_exception_missing_name(self):
        store_plugin = 'store_crypto'
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretStores, None, store_plugin)
        self.assertRaises(exception.MissingArgumentError,
                          models.SecretStores, "", store_plugin)

    def test_secret_stores_check_to_dict_fields(self):
        name = "pkcs11 backend"
        store_plugin = 'store_crypto'
        crypto_plugin = 'p11_crypto'
        ss = models.SecretStores(name, store_plugin, crypto_plugin,
                                 global_default=True)
        self.assertEqual(store_plugin,
                         ss.to_dict_fields()['store_plugin'])
        self.assertEqual(crypto_plugin,
                         ss.to_dict_fields()['crypto_plugin'])
        self.assertTrue(ss.to_dict_fields()['global_default'])
        self.assertEqual(models.States.ACTIVE,
                         ss.to_dict_fields()['status'])
        self.assertEqual(name, ss.to_dict_fields()['name'])

        # check with required input only
        ss = models.SecretStores(name, store_plugin)
        self.assertEqual(store_plugin,
                         ss.to_dict_fields()['store_plugin'])
        self.assertIsNone(ss.to_dict_fields()['crypto_plugin'])
        self.assertIsNone(ss.to_dict_fields()['global_default'])
        self.assertEqual(models.States.ACTIVE,
                         ss.to_dict_fields()['status'])
        self.assertEqual(name, ss.to_dict_fields()['name'])


class WhenCreatingNewProjectSecretStore(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewProjectSecretStore, self).setUp()

    def test_new_project_secret_store(self):

        project_id = 'proj_123456'
        name = "db backend"
        store_plugin = 'store_crypto'
        crypto_plugin = 'simple_crypto'
        ss = models.SecretStores(name, store_plugin, crypto_plugin,
                                 global_default=True)
        ss.id = "ss_123456"

        project_ss = models.ProjectSecretStore(project_id, ss.id)
        self.assertEqual(project_id, project_ss.project_id)
        self.assertEqual(ss.id, project_ss.secret_store_id)
        self.assertEqual(models.States.ACTIVE, project_ss.status)

    def test_should_throw_exception_missing_project_id(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.ProjectSecretStore, None, "ss_123456")
        self.assertRaises(exception.MissingArgumentError,
                          models.ProjectSecretStore, "", "ss_123456")

    def test_should_throw_exception_missing_secret_store_id(self):
        self.assertRaises(exception.MissingArgumentError,
                          models.ProjectSecretStore, "proj_123456", None)
        self.assertRaises(exception.MissingArgumentError,
                          models.ProjectSecretStore, "proj_123456", "")

    def test_project_secret_store_check_to_dict_fields(self):
        project_id = 'proj_123456'
        secret_store_id = 'ss_7689012'
        project_ss = models.ProjectSecretStore(project_id, secret_store_id)
        self.assertEqual(project_id,
                         project_ss.to_dict_fields()['project_id'])
        self.assertEqual(secret_store_id,
                         project_ss.to_dict_fields()['secret_store_id'])
        self.assertEqual(models.States.ACTIVE,
                         project_ss.to_dict_fields()['status'])


class WhenCreatingNewSecretConsumer(utils.BaseTestCase):
    def setUp(self):
        super(WhenCreatingNewSecretConsumer, self).setUp()
        self.secret_id = "12345secret"
        self.project_id = "12345project"
        self.service = "12345service"
        self.resource_type = "12345resource_type"
        self.resource_id = "12345resource_id"

    def test_new_secret_consumer(self):
        consumer = models.SecretConsumerMetadatum(
            self.secret_id,
            self.project_id,
            self.service,
            self.resource_type,
            self.resource_id
        )
        self.assertEqual(self.secret_id, consumer.secret_id)
        self.assertEqual(self.project_id, consumer.project_id)
        self.assertEqual(self.service, consumer.service)
        self.assertEqual(self.resource_type, consumer.resource_type)
        self.assertEqual(self.resource_id, consumer.resource_id)
        self.assertEqual(models.States.ACTIVE, consumer.status)

    def test_to_dict_fields(self):
        consumer = models.SecretConsumerMetadatum(
            self.secret_id,
            self.project_id,
            self.service,
            self.resource_type,
            self.resource_id
        )
        fields = consumer.to_dict_fields()
        self.assertEqual(self.service, fields["service"])
        self.assertEqual(self.resource_type, fields["resource_type"])
        self.assertEqual(self.resource_id, fields["resource_id"])

    def test_should_raise_exception_when_missing_arguments(self):
        self.assertRaises(
            exception.MissingArgumentError,
            models.SecretConsumerMetadatum,
            None,
            self.project_id,
            self.service,
            self.resource_type,
            self.resource_id,
        )
        self.assertRaises(
            exception.MissingArgumentError,
            models.SecretConsumerMetadatum,
            self.secret_id,
            None,
            self.service,
            self.resource_type,
            self.resource_id,
        )
        self.assertRaises(
            exception.MissingArgumentError,
            models.SecretConsumerMetadatum,
            self.secret_id,
            self.project_id,
            None,
            self.resource_type,
            self.resource_id,
        )
        self.assertRaises(
            exception.MissingArgumentError,
            models.SecretConsumerMetadatum,
            self.secret_id,
            self.project_id,
            self.service,
            None,
            self.resource_id,
        )
        self.assertRaises(
            exception.MissingArgumentError,
            models.SecretConsumerMetadatum,
            self.secret_id,
            self.project_id,
            self.service,
            self.resource_type,
            None,
        )


if __name__ == '__main__':
    unittest.main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime

from barbican.common import exception
from barbican.common import resources as res
from barbican.model import models
from barbican.model import repositories
from barbican.tests import database_utils


class WhenTestingCertificateAuthorityRepo(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingCertificateAuthorityRepo, self).setUp()
        self.ca_repo = repositories.CertificateAuthorityRepo()
        expiration = (datetime.datetime.utcnow() +
                      datetime.timedelta(minutes=10))
        self.parsed_ca = {'plugin_name': 'dogtag_plugin',
                          'plugin_ca_id': 'ca_master',
                          'name': 'Dogtag CA',
                          'expiration': expiration.isoformat(),
                          'description': 'Master CA for Dogtag plugin',
                          'ca_signing_certificate': 'XXXXX',
                          'intermediates': 'YYYYY'}
        self.parsed_ca2 = {'plugin_name': 'symantec_plugin',
                           'plugin_ca_id': 'ca_master_2',
                           'name': 'Symantec CA2',
                           'expiration': expiration.isoformat(),
                           'description': 'Master CA for Dogtag plugin2',
                           'ca_signing_certificate': 'XXXXX',
                           'intermediates': 'YYYYY'}

    def _add_ca(self, parsed_ca, session):
        ca = self.ca_repo.create_from(models.CertificateAuthority(parsed_ca),
                                      session=session)
        return ca

    def test_get_by_create_date(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        session.commit()

        retrieved_cas, offset, limit, total = self.ca_repo.get_by_create_date(
            session=session
        )

        self.assertEqual([ca.id], [s.id for s in retrieved_cas])
        self.assertEqual([ca.plugin_name],
                         [s.plugin_name for s in retrieved_cas])
        self.assertEqual(
            [self.parsed_ca['ca_signing_certificate']],
            [s.ca_meta['ca_signing_certificate'].value for s in retrieved_cas])
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(1, total)

    def test_get_by_create_date_with_plugin_name_filter(self):
        session = self.ca_repo.get_session()
        ca1 = self._add_ca(self.parsed_ca, session)
        self._add_ca(self.parsed_ca2, session)

        retrieved_cas, offset, limit, total = self.ca_repo.get_by_create_date(
            session=session,
            plugin_name=self.parsed_ca['plugin_name']
        )

        self.assertEqual([ca1.id], [s.id for s in retrieved_cas])
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(1, total)

    def test_get_by_create_date_with_plugin_ca_id_filter(self):
        session = self.ca_repo.get_session()
        ca1 = self._add_ca(self.parsed_ca, session)
        self._add_ca(self.parsed_ca2, session)

        retrieved_cas, offset, limit, total = self.ca_repo.get_by_create_date(
            session=session,
            plugin_ca_id=self.parsed_ca['plugin_ca_id']
        )

        self.assertEqual([ca1.id], [s.id for s in retrieved_cas])
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(1, total)

    def test_get_by_create_date_nothing(self):
        session = self.ca_repo.get_session()
        retrieved_cas, offset, limit, total = self.ca_repo.get_by_create_date(
            session=session,
            suppress_exception=True
        )

        self.assertEqual([], retrieved_cas)
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(0, total)

    def test_do_entity_name(self):
        self.assertEqual("CertificateAuthority",
                         self.ca_repo._do_entity_name())

    def test_should_raise_no_result_found(self):
        session = self.ca_repo.get_session()

        self.assertRaises(
            exception.NotFound,
            self.ca_repo.get_by_create_date,
            session=session,
            suppress_exception=False)

    def test_get_count_should_return_zero(self):
        session = self.ca_repo.get_session()

        project = models.Project()
        project.external_id = "my keystone id"
        project.save(session=session)

        session.commit()
        count = self.ca_repo.get_count(project.id, session=session)

        self.assertEqual(0, count)

    def test_get_count_should_return_one(self):
        session = self.ca_repo.get_session()

        project = models.Project()
        project.external_id = "my keystone id"
        project.save(session=session)

        ca_model = models.CertificateAuthority(self.parsed_ca)
        ca_model.project_id = project.id
        self.ca_repo.create_from(ca_model, session=session)

        session.commit()
        count = self.ca_repo.get_count(project.id, session=session)

        self.assertEqual(1, count)

    def test_get_count_should_return_one_after_delete(self):
        session = self.ca_repo.get_session()

        project = models.Project()
        project.external_id = "my keystone id"
        project.save(session=session)

        ca_model = models.CertificateAuthority(self.parsed_ca)
        ca_model.project_id = project.id
        self.ca_repo.create_from(ca_model, session=session)

        ca_model = models.CertificateAuthority(self.parsed_ca)
        ca_model.project_id = project.id
        self.ca_repo.create_from(ca_model, session=session)

        session.commit()
        count = self.ca_repo.get_count(project.id, session=session)
        self.assertEqual(2, count)

        self.ca_repo.delete_entity_by_id(ca_model.id, "my keystone id",
                                         session=session)
        session.commit()

        count = self.ca_repo.get_count(project.id, session=session)
        self.assertEqual(1, count)


class WhenTestingProjectCARepo(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingProjectCARepo, self).setUp()
        self.ca_repo = repositories.CertificateAuthorityRepo()
        self.project_ca_repo = repositories.ProjectCertificateAuthorityRepo()
        expiration = (datetime.datetime.utcnow() +
                      datetime.timedelta(minutes=10))
        self.parsed_ca = {'plugin_name': 'dogtag_plugin',
                          'plugin_ca_id': 'ca_master',
                          'expiration': expiration.isoformat(),
                          'name': 'Dogtag CA',
                          'description': 'Master CA for Dogtag plugin',
                          'ca_signing_certificate': 'XXXXX',
                          'intermediates': 'YYYYY'}
        self.parsed_ca2 = {'plugin_name': 'symantec_plugin',
                           'plugin_ca_id': 'ca_master_2',
                           'expiration': expiration.isoformat(),
                           'name': 'Symantec CA2',
                           'description': 'Master CA for Dogtag plugin2',
                           'ca_signing_certificate': 'XXXXX',
                           'intermediates': 'YYYYY'}

    def _add_ca(self, parsed_ca, session):
        ca = self.ca_repo.create_from(models.CertificateAuthority(parsed_ca),
                                      session=session)
        return ca

    def _add_project(self, project_id, session):
        project = models.Project()
        project.external_id = project_id
        project.save(session=session)
        return project

    def _add_project_ca(self, project_id, ca_id, session):
        project_ca = self.project_ca_repo.create_from(
            models.ProjectCertificateAuthority(project_id, ca_id),
            session)
        return project_ca

    def test_get_by_create_date(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        project = self._add_project("project_1", session)
        self._add_project_ca(project.id, ca.id, session)

        session.commit()

        retrieved_pcas, offset, limit, total = (
            self.project_ca_repo.get_by_create_date(session=session))

        self.assertEqual([ca.id], [s.ca_id for s in retrieved_pcas])
        self.assertEqual([project.id], [s.project_id for s in retrieved_pcas])
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(1, total)

    def test_get_project_entities(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        project = self._add_project("project_1", session)
        self._add_project_ca(project.id, ca.id, session)

        session.commit()

        retrieved_pcas = self.project_ca_repo.get_project_entities(
            project.id, session)
        self.assertEqual([ca.id], [s.ca_id for s in retrieved_pcas])

    def test_get_by_create_date_with_ca_id_filter(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        project = self._add_project("project_1", session)
        project_ca = self._add_project_ca(project.id, ca.id, session)

        ca2 = self._add_ca(self.parsed_ca2, session)
        project2 = self._add_project("project_2", session)
        self._add_project_ca(project2.id, ca2.id, session)

        session.commit()

        retrieved_pcas, offset, limit, total = (
            self.project_ca_repo.get_by_create_date(
                session=session,
                ca_id=ca.id))

        self.assertEqual([project_ca.id],
                         [s.id for s in retrieved_pcas])
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(1, total)

    def test_get_by_create_date_nothing(self):
        session = self.project_ca_repo.get_session()
        retrieved_pcas, offset, limit, total = (
            self.project_ca_repo.get_by_create_date(
                session=session,
                suppress_exception=True))

        self.assertEqual([], retrieved_pcas)
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(0, total)

    def test_do_entity_name(self):
        self.assertEqual("ProjectCertificateAuthority",
                         self.project_ca_repo._do_entity_name())

    def test_should_raise_no_result_found(self):
        session = self.project_ca_repo.get_session()

        self.assertRaises(
            exception.NotFound,
            self.project_ca_repo.get_by_create_date,
            session=session,
            suppress_exception=False)


class WhenTestingPreferredCARepo(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingPreferredCARepo, self).setUp()
        self.ca_repo = repositories.CertificateAuthorityRepo()
        self.preferred_ca_repo = (
            repositories.PreferredCertificateAuthorityRepo())

        expiration = (datetime.datetime.utcnow() +
                      datetime.timedelta(minutes=10))

        expiration_later = (datetime.datetime.utcnow() +
                            datetime.timedelta(days=10))
        self.parsed_ca = {'plugin_name': 'dogtag_plugin',
                          'plugin_ca_id': 'ca_master',
                          'expiration': expiration.isoformat(),
                          'name': 'Dogtag CA',
                          'description': 'Master CA for Dogtag plugin',
                          'ca_signing_certificate': 'XXXXX',
                          'intermediates': 'YYYYY'}

        self.parsed_ca2 = {'plugin_name': 'symantec_plugin',
                           'plugin_ca_id': 'ca_master_2',
                           'expiration': expiration.isoformat(),
                           'name': 'Symantec CA2',
                           'description': 'Master CA for Dogtag plugin2',
                           'ca_signing_certificate': 'XXXXX',
                           'intermediates': 'YYYYY'}

        self.parsed_modified_ca = {
            'plugin_name': 'dogtag_plugin',
            'plugin_ca_id': 'ca_master',
            'expiration': expiration_later.isoformat(),
            'name': 'Dogtag CA',
            'description': 'Updated Master CA for Dogtag plugin',
            'ca_signing_certificate': 'XXXXX-updated-XXXXX',
            'intermediates': 'YYYYY'}

        self.global_project = res.get_or_create_global_preferred_project()

    def _add_ca(self, parsed_ca, session):
        ca = self.ca_repo.create_from(models.CertificateAuthority(parsed_ca),
                                      session=session)
        return ca

    def _add_project(self, project_id, session):
        project = models.Project()
        project.external_id = project_id
        project.save(session=session)
        return project

    def _add_preferred_ca(self, project_id, ca_id, session):
        preferred_ca = self.preferred_ca_repo.create_from(
            models.PreferredCertificateAuthority(project_id, ca_id),
            session)
        return preferred_ca

    def _add_global_preferred_ca(self, ca_id, session):
        preferred_ca = self.preferred_ca_repo.create_from(
            models.PreferredCertificateAuthority(
                self.global_project.id,
                ca_id),
            session)
        return preferred_ca

    def test_get_by_create_date(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        self._add_ca(self.parsed_ca2, session)
        project = self._add_project("project_1", session)
        self._add_preferred_ca(project.id, ca.id, session)

        session.commit()

        pca, offset, limit, total = self.preferred_ca_repo.get_by_create_date(
            session=session
        )

        self.assertEqual([ca.id], [s.ca_id for s in pca])
        self.assertEqual([project.id], [s.project_id for s in pca])
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(1, total)

    def test_get_by_create_date_with_params(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        self._add_ca(self.parsed_ca2, session)
        project = self._add_project("project_1", session)
        self._add_preferred_ca(project.id, ca.id, session)

        session.commit()

        pca, offset, limit, total = self.preferred_ca_repo.get_by_create_date(
            session=session,
            project_id=project.id,
            ca_id=ca.id
        )

        self.assertEqual([ca.id], [s.ca_id for s in pca])
        self.assertEqual([project.id], [s.project_id for s in pca])
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(1, total)

    def test_get_project_entities(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        self._add_ca(self.parsed_ca2, session)
        project = self._add_project("project_1", session)
        self._add_preferred_ca(project.id, ca.id, session)

        session.commit()
        pca = self.preferred_ca_repo.get_project_entities(project.id, session)
        self.assertEqual([ca.id], [s.ca_id for s in pca])

    def test_get_nothing(self):
        session = self.preferred_ca_repo.get_session()
        retrieved_pcas, offset, limit, total = (
            self.preferred_ca_repo.get_by_create_date(
                session=session,
                suppress_exception=True))

        self.assertEqual([], retrieved_pcas)
        self.assertEqual(0, offset)
        self.assertEqual(10, limit)
        self.assertEqual(0, total)

    def test_do_entity_name(self):
        self.assertEqual("PreferredCertificateAuthority",
                         self.preferred_ca_repo._do_entity_name())

    def test_should_raise_no_result_found(self):
        session = self.preferred_ca_repo.get_session()

        self.assertRaises(
            exception.NotFound,
            self.preferred_ca_repo.get_by_create_date,
            session=session,
            suppress_exception=False)

    def test_should_raise_constraint_check(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        ca2 = self._add_ca(self.parsed_ca2, session)
        project = self._add_project("project_1", session)
        self._add_preferred_ca(project.id, ca.id, session)
        self.assertRaises(
            exception.ConstraintCheck,
            self._add_preferred_ca,
            project.id,
            ca2.id,
            session)

    def test_set_global_preferred_ca(self):
        session = self.ca_repo.get_session()

        ca = self._add_ca(self.parsed_ca, session)
        self._add_global_preferred_ca(ca.id, session)
        session.commit()

        pca = self.preferred_ca_repo.get_project_entities(
            self.global_project.id,
            session)
        self.assertEqual([ca.id], [s.ca_id for s in pca])

    def test_should_create(self):
        session = self.ca_repo.get_session()
        ca = self._add_ca(self.parsed_ca, session)
        project = self._add_project("project_1", session)

        self.preferred_ca_repo.create_or_update_by_project_id(
            project.id, ca.id)
        session.commit()

    def test_should_update(self):
        session = self.ca_repo.get_session()
        ca1 = self._add_ca(self.parsed_ca, session)
        ca2 = self._add_ca(self.parsed_ca2, session)
        project = self._add_project("project_1", session)

        self.preferred_ca_repo.create_or_update_by_project_id(
            project.id, ca1.id)
        session.commit()
        self.preferred_ca_repo.create_or_update_by_project_id(
            project.id, ca2.id)
        session.commit()



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import time

from barbican.common import config
from barbican.common import exception
from barbican.model import models
from barbican.model import repositories
from barbican.tests import database_utils


class WhenTestingOrderRetryTaskRepository(database_utils.RepositoryTestCase):

    def setUp(self):
        super(WhenTestingOrderRetryTaskRepository, self).setUp()

        self.date_time_now = datetime.datetime.utcnow()
        self.test_args = ['test', 'args']
        self.test_kwargs = {'test': 1, 'kwargs': 2}

        self.repo = repositories.OrderRetryTaskRepo()
        self.order_repo = repositories.OrderRepo()

    def test_get_order_retry_task(self):
        session = self.repo.get_session()

        order_retry_task = self._create_retry_task(session)

        order_retry_task_from_get = self.repo.get(
            order_retry_task.id,
            session=session,
        )

        self.assertEqual(order_retry_task.id, order_retry_task_from_get.id)
        self.assertEqual(
            self.date_time_now, order_retry_task_from_get.retry_at)
        self.assertEqual(u'retry-task', order_retry_task_from_get.retry_task)
        self.assertEqual(self.test_args, order_retry_task_from_get.retry_args)
        self.assertEqual(self.test_kwargs,
                         order_retry_task_from_get.retry_kwargs)

    def test_get_order_retry_task_filtered_by_retry_time(self):
        session = self.repo.get_session()

        future_seconds = 3
        date_time_future = (
            self.date_time_now + datetime.timedelta(seconds=future_seconds)
        )

        order_retry_task = self._create_retry_task(
            session, retry_at=date_time_future)

        # A retrieve by the current time should return no entries, as the only
        # retry record is set into the future.
        entities, offset, limit, total = self.repo.get_by_create_date(
            only_at_or_before_this_date=self.date_time_now,
            session=session,
            suppress_exception=True
        )
        self.assertEqual(0, total)
        self.assertEqual([], entities)

        # Wait until the future time is the current time.
        time.sleep(2 * future_seconds)

        # Now, a retrieve by the current time should return our entry.
        entities, offset, limit, total = self.repo.get_by_create_date(
            only_at_or_before_this_date=datetime.datetime.utcnow(),
            session=session,
            suppress_exception=True
        )
        self.assertEqual(1, total)

        # Verify that retry task record is what we put in originally.
        order_retry_task_from_get = entities[0]
        self.assertEqual(order_retry_task.id, order_retry_task_from_get.id)
        self.assertEqual(date_time_future, order_retry_task_from_get.retry_at)
        self.assertEqual(u'retry-task', order_retry_task_from_get.retry_task)
        self.assertEqual(self.test_args, order_retry_task_from_get.retry_args)
        self.assertEqual(self.test_kwargs,
                         order_retry_task_from_get.retry_kwargs)

    def test_should_raise_no_result_found_no_exception(self):
        session = self.repo.get_session()

        entities, offset, limit, total = self.repo.get_by_create_date(
            session=session,
            suppress_exception=True)

        self.assertEqual([], entities)
        self.assertEqual(0, offset)
        self.assertEqual(config.CONF.default_limit_paging, limit)
        self.assertEqual(0, total)

    def test_should_raise_no_result_found_with_exceptions(self):
        session = self.repo.get_session()

        self.assertRaises(
            exception.NotFound,
            self.repo.get_by_create_date,
            session=session,
            suppress_exception=False)

    def _create_retry_task(self, session, retry_at=None):
        project = database_utils.create_project(session=session)
        order = database_utils.create_order(project, session=session)

        order_retry_task = models.OrderRetryTask()
        order_retry_task.order_id = order.id
        order_retry_task.retry_task = u'retry-task'
        order_retry_task.retry_at = retry_at or self.date_time_now
        order_retry_task.retry_args = self.test_args
        order_retry_task.retry_kwargs = self.test_kwargs
        self.repo.create_from(order_retry_task, session=session)

        session.commit()

        return order_retry_task



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from unittest import mock

import fixtures

from barbican.cmd import barbican_manage as manager
from barbican.tests import utils


class TestBarbicanManageBase(utils.BaseTestCase):
    def setUp(self):
        super(TestBarbicanManageBase, self).setUp()

        def clear_conf():
            manager.CONF.reset()
            manager.CONF.unregister_opt(manager.category_opt)
        clear_conf()
        self.addCleanup(clear_conf)

        self.useFixture(fixtures.MonkeyPatch(
            'oslo_log.log.setup', lambda barbican_test, version='test': None))
        manager.CONF.set_override('sql_connection', 'mockdburl')

    def _main_test_helper(self, argv, func_name=None, *exp_args, **exp_kwargs):
        self.useFixture(fixtures.MonkeyPatch('sys.argv', argv))
        manager.main()
        func_name.assert_called_once_with(*exp_args, **exp_kwargs)


class TestBarbicanManage(TestBarbicanManageBase):
    """Test barbican-manage functionality."""

    @mock.patch('barbican.model.migration.commands.generate')
    def test_db_revision(self, mock_generate):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'revision', '--db-url',
             'mockdb', '--message', 'mockmsg'], mock_generate,
            autogenerate=False, message='mockmsg', sql_url='mockdb')

    @mock.patch('barbican.model.migration.commands.generate')
    def test_db_revision_autogenerate(self, mock_generate):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'revision', '--db-url',
             'mockdb', '--message', 'mockmsg', '--autogenerate'],
            mock_generate, autogenerate=True, message='mockmsg',
            sql_url='mockdb')

    @mock.patch('barbican.model.migration.commands.generate')
    def test_db_revision_no_dburl(self, mock_generate):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'revision', '--message',
             'mockmsg'], mock_generate, autogenerate=False, message='mockmsg',
            sql_url='mockdburl')

    @mock.patch('barbican.model.migration.commands.upgrade')
    def test_db_upgrade(self, mock_upgrade):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'upgrade', '--db-url',
             'mockdb'], mock_upgrade, to_version='head', sql_url='mockdb')

    @mock.patch('barbican.model.migration.commands.upgrade')
    def test_db_upgrade_no_dburl(self, mock_upgrade):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'upgrade'], mock_upgrade,
            to_version='head', sql_url='mockdburl')

    @mock.patch('barbican.model.migration.commands.history')
    def test_db_history(self, mock_history):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'history', '--db-url',
             'mockdb'], mock_history, False, sql_url='mockdb')

    @mock.patch('barbican.model.migration.commands.history')
    def test_db_history_no_dburl(self, mock_history):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'history'], mock_history,
            False, sql_url='mockdburl')

    @mock.patch('barbican.model.clean.clean_command')
    def test_db_clean_no_args(self, mock_clean_command):
        manager.CONF.set_override('log_file', 'mock_log_file')
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'clean'],
            func_name=mock_clean_command,
            sql_url='mockdburl',
            min_num_days=90,
            do_clean_unassociated_projects=False,
            do_soft_delete_expired_secrets=False,
            verbose=False,
            log_file='mock_log_file')
        manager.CONF.clear_override('log_file')

    @mock.patch('barbican.model.clean.clean_command')
    def test_db_clean_with_args(self, mock_clean_command):
        manager.CONF.set_override('log_file', 'mock_log_file')
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'clean',
             '--db-url', 'somewhere', '--min-days', '180',
             '--clean-unassociated-projects', '--soft-delete-expired-secrets',
             '--verbose', '--log-file', '/tmp/whatevs'],
            func_name=mock_clean_command,
            sql_url='somewhere',
            min_num_days=180,
            do_clean_unassociated_projects=True,
            do_soft_delete_expired_secrets=True,
            verbose=True,
            log_file='/tmp/whatevs')
        manager.CONF.clear_override('log_file')

    @mock.patch('barbican.model.sync.sync_secret_stores')
    def test_db_sync_secret_stores_no_args(self, mock_sync_command):
        manager.CONF.set_override('log_file', 'mock_log_file')
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'sync_secret_stores'],
            func_name=mock_sync_command,
            sql_url='mockdburl',
            verbose=False,
            log_file='mock_log_file')
        manager.CONF.clear_override('log_file')

    @mock.patch('barbican.model.sync.sync_secret_stores')
    def test_db_sync_secret_stores_with_args(self, mock_sync_command):
        manager.CONF.set_override('log_file', 'mock_log_file')
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'sync_secret_stores',
             '--db-url', 'somewhere',
             '--verbose',
             '--log-file', '/tmp/whatevs'],
            func_name=mock_sync_command,
            sql_url='somewhere',
            verbose=True,
            log_file='/tmp/whatevs')
        manager.CONF.clear_override('log_file')

    @mock.patch('barbican.model.migration.commands.current')
    def test_db_current(self, mock_current):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'current', '--db-url',
             'mockdb'], mock_current, False, sql_url='mockdb')

    @mock.patch('barbican.model.migration.commands.current')
    def test_db_current_no_dburl(self, mock_current):
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'db', 'current'], mock_current,
            False, sql_url='mockdburl')

    @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
    def test_hsm_check_mkek(self, mock_pkcs11):
        mock_pkcs11.return_value.get_session.return_value = 1
        mock_pkcs11.return_value.get_key_handle.return_value = 1
        mock_getkey = mock_pkcs11.return_value.get_key_handle
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'hsm', 'check_mkek',
             '--library-path', 'mocklib', '--passphrase', 'mockpassewd',
             '--label', 'mocklabel'], mock_getkey, 'CKK_AES', 'mocklabel', 1)

    @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
    def test_hsm_gen_mkek(self, mock_pkcs11):
        mock_pkcs11.return_value.get_session.return_value = 1
        mock_pkcs11.return_value.get_key_handle.return_value = None
        mock_pkcs11.return_value.generate_key.return_value = 0
        mock_genkey = mock_pkcs11.return_value.generate_key
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'hsm', 'gen_mkek',
             '--library-path', 'mocklib', '--passphrase', 'mockpassewd',
             '--label', 'mocklabel'], mock_genkey, 'CKK_AES',
            32, 'CKM_AES_KEY_GEN', 1, 'mocklabel', encrypt=True, wrap=True,
            master_key=True)

    @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
    def test_hsm_gen_hmac(self, mock_pkcs11):
        mock_pkcs11.return_value.get_session.return_value = 1
        mock_pkcs11.return_value.get_key_handle.return_value = None
        mock_pkcs11.return_value.generate_key.return_value = 0
        mock_genkey = mock_pkcs11.return_value.generate_key
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'hsm', 'gen_hmac',
             '--library-path', 'mocklib', '--passphrase', 'mockpassewd',
             '--label', 'mocklabel'], mock_genkey, 'CKK_AES',
            32, 'CKM_AES_KEY_GEN', 1, 'mocklabel', sign=True, master_key=True)

    @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
    def test_hsm_gen_mkek_non_default_length(self, mock_pkcs11):
        mock_pkcs11.return_value.get_session.return_value = 1
        mock_pkcs11.return_value.get_key_handle.return_value = None
        mock_pkcs11.return_value.generate_key.return_value = 0
        mock_genkey = mock_pkcs11.return_value.generate_key
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'hsm', 'gen_mkek',
             '--length', '48', '--library-path', 'mocklib',
             '--passphrase', 'mockpassewd', '--label', 'mocklabel'],
            mock_genkey, 'CKK_AES', 48, 'CKM_AES_KEY_GEN', 1, 'mocklabel',
            encrypt=True, wrap=True, master_key=True)

    @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
    def test_hsm_check_hmac(self, mock_pkcs11):
        mock_pkcs11.return_value.get_session.return_value = 1
        mock_pkcs11.return_value.get_key_handle.return_value = 1
        mock_getkey = mock_pkcs11.return_value.get_key_handle
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'hsm', 'check_hmac',
             '--library-path', 'mocklib', '--passphrase', 'mockpassewd',
             '--label', 'mocklabel'], mock_getkey, 'CKK_AES', 'mocklabel', 1)

    @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
    def test_hsm_gen_hmac_non_default_length(self, mock_pkcs11):
        mock_pkcs11.return_value.get_session.return_value = 1
        mock_pkcs11.return_value.get_key_handle.return_value = None
        mock_pkcs11.return_value.generate_key.return_value = 0
        mock_genkey = mock_pkcs11.return_value.generate_key
        self._main_test_helper(
            ['barbican.cmd.barbican_manage', 'hsm', 'gen_hmac',
             '--length', '48', '--library-path', 'mocklib',
             '--passphrase', 'mockpassewd', '--label', 'mocklabel'],
            mock_genkey, 'CKK_AES', 48, 'CKM_AES_KEY_GEN', 1, 'mocklabel',
            sign=True, master_key=True)



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_versionedobjects import base as object_base

from barbican.model import models
from barbican.model import repositories as repo
from barbican.objects import base
from barbican.objects import fields


@object_base.VersionedObjectRegistry.register
class EncryptedDatum(base.BarbicanObject, base.BarbicanPersistentObject,
                     object_base.VersionedObjectDictCompat):

    fields = {
        'content_type': fields.StringField(nullable=True, default=None),
        'secret_id': fields.StringField(),
        'kek_id': fields.StringField(),
        'cypher_text': fields.StringField(nullable=True, default=None),
        'kek_meta_extended': fields.StringField(nullable=True, default=None),
        'kek_meta_project': fields.ObjectField('KEKDatum',
                                               nullable=True, default=None),
        'status': fields.StringField(nullable=True, default=base.States.ACTIVE)
    }

    db_model = models.EncryptedDatum
    db_repo = repo.get_encrypted_datum_repository()
    synthetic_fields = ['kek_meta_project']



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from six.moves import http_client


host = "localhost"
port = 9311
method = "GET"
timeout = 1000
body = None
path = "/"
headers = ""

expected_response = {"v1": "current", "build": "0.1.34dev"}


# Typically an authenticated user session will make a request for a key to
# barbican
# The restful request in all likelihood contain an auth token
# this test mimics such a request provided a token

# if pki tokens are used, the token is rather large
# uuid tokens are smaller and easier to test with
# assume there is a "demo" user with only member role

# curl -XPOST -d '{"auth":{"passwordCredentials":{"username": "demo",
# "password": "secret"}, "tenantName": "demo"}}'
# -H "Content-type: application/json" http://localhost:5000/v3/tokens
#
# pull out the token_id from above and use in ping_barbican
#

# TODO(malini) flesh this out
def get_demo_token(password):
    pass


def ping_barbican(token_id):
    headers = {'X_AUTH_TOKEN': token_id, 'X_IDENTITY_STATUS': 'Confirmed'}
    connection = http_client.HTTPConnection(host, port, timeout=timeout)
    connection.request(method, path, None, headers)
    response = connection.getresponse().read()
    connection.close()
    return response



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
import collections
import pecan

from barbican import api
from barbican.api import controllers
from barbican.common import hrefs
from barbican.common import utils
from barbican.common import validators
from barbican import i18n as u
from barbican.model import repositories as repo

LOG = utils.getLogger(__name__)


def _secret_metadata_not_found():
    """Throw exception indicating secret metadata not found."""
    pecan.abort(404, u._('Secret metadata not found.'))


class SecretMetadataController(controllers.ACLMixin):
    """Handles SecretMetadata requests by a given secret id."""

    def __init__(self, secret):
        LOG.debug('=== Creating SecretMetadataController ===')
        self.secret = secret
        self.secret_project_id = self.secret.project.external_id
        self.secret_repo = repo.get_secret_repository()
        self.user_meta_repo = repo.get_secret_user_meta_repository()
        self.metadata_validator = validators.NewSecretMetadataValidator()
        self.metadatum_validator = validators.NewSecretMetadatumValidator()

    @pecan.expose(generic=True)
    def index(self, **kwargs):
        pecan.abort(405)  # HTTP 405 Method Not Allowed as default

    @index.when(method='GET', template='json')
    @utils.allow_all_content_types
    @controllers.handle_exceptions(u._('Secret metadata retrieval'))
    @controllers.enforce_rbac('secret_meta:get')
    def on_get(self, external_project_id, **kwargs):
        """Handles retrieval of existing secret metadata requests."""

        LOG.debug('Start secret metadata on_get '
                  'for secret-ID %s:', self.secret.id)

        resp = self.user_meta_repo.get_metadata_for_secret(self.secret.id)
        pecan.response.status = 200

        return {"metadata": resp}

    @index.when(method='PUT', template='json')
    @controllers.handle_exceptions(u._('Secret metadata creation'))
    @controllers.enforce_rbac('secret_meta:put')
    @controllers.enforce_content_types(['application/json'])
    def on_put(self, external_project_id, **kwargs):
        """Handles creation/update of secret metadata."""
        data = api.load_body(pecan.request, validator=self.metadata_validator)
        LOG.debug('Start secret metadata on_put...%s', data)

        self.user_meta_repo.create_replace_user_metadata(self.secret.id,
                                                         data)

        url = hrefs.convert_user_meta_to_href(self.secret.id)
        LOG.debug('URI to secret metadata is %s', url)

        pecan.response.status = 201
        return {'metadata_ref': url}

    @index.when(method='POST', template='json')
    @controllers.handle_exceptions(u._('Secret metadatum creation'))
    @controllers.enforce_rbac('secret_meta:post')
    @controllers.enforce_content_types(['application/json'])
    def on_post(self, external_project_id, **kwargs):
        """Handles creation of secret metadatum."""

        data = api.load_body(pecan.request, validator=self.metadatum_validator)

        key = data.get('key')
        value = data.get('value')

        metadata = self.user_meta_repo.get_metadata_for_secret(self.secret.id)
        if key in metadata:
            pecan.abort(409, u._('Conflict. Key in request is already in the '
                                 'secret metadata'))

        LOG.debug('Start secret metadatum on_post...%s', metadata)
        self.user_meta_repo.create_replace_user_metadatum(self.secret.id,
                                                          key, value)

        url = hrefs.convert_user_meta_to_href(self.secret.id)
        LOG.debug('URI to secret metadata is %s', url)

        pecan.response.status = 201
        return {'metadata_ref': url + "/%s {key: %s, value:%s}" % (key,
                                                                   key,
                                                                   value)}


class SecretMetadatumController(controllers.ACLMixin):

    def __init__(self, secret):
        LOG.debug('=== Creating SecretMetadatumController ===')
        self.user_meta_repo = repo.get_secret_user_meta_repository()
        self.secret = secret
        self.metadatum_validator = validators.NewSecretMetadatumValidator()

    @pecan.expose(generic=True)
    def index(self, **kwargs):
        pecan.abort(405)  # HTTP 405 Method Not Allowed as default

    @index.when(method='GET', template='json')
    @controllers.handle_exceptions(u._('Secret metadatum retrieval'))
    @controllers.enforce_rbac('secret_meta:get')
    def on_get(self, external_project_id, remainder, **kwargs):
        """Handles retrieval of existing secret metadatum."""

        LOG.debug('Start secret metadatum on_get '
                  'for secret-ID %s:', self.secret.id)

        metadata = self.user_meta_repo.get_metadata_for_secret(self.secret.id)
        if remainder in metadata:
            pecan.response.status = 200
            pair = {'key': remainder, 'value': metadata[remainder]}
            return collections.OrderedDict(sorted(pair.items()))
        else:
            _secret_metadata_not_found()

    @index.when(method='PUT', template='json')
    @utils.allow_all_content_types
    @controllers.handle_exceptions(u._('Secret metadatum update'))
    @controllers.enforce_rbac('secret_meta:put')
    @controllers.enforce_content_types(['application/json'])
    def on_put(self, external_project_id, remainder, **kwargs):
        """Handles update of existing secret metadatum."""
        metadata = self.user_meta_repo.get_metadata_for_secret(self.secret.id)
        data = api.load_body(pecan.request, validator=self.metadatum_validator)

        key = data.get('key')
        value = data.get('value')

        if remainder not in metadata:
            _secret_metadata_not_found()
        elif remainder != key:
            msg = 'Key in request data does not match key in the '
            'request url.'
            pecan.abort(409, msg)
        else:
            LOG.debug('Start secret metadatum on_put...%s', metadata)

            self.user_meta_repo.create_replace_user_metadatum(self.secret.id,
                                                              key, value)

            pecan.response.status = 200
            pair = {'key': key, 'value': value}
            return collections.OrderedDict(sorted(pair.items()))

    @index.when(method='DELETE', template='json')
    @controllers.handle_exceptions(u._('Secret metadatum removal'))
    @controllers.enforce_rbac('secret_meta:delete')
    def on_delete(self, external_project_id, remainder, **kwargs):
        """Handles removal of existing secret metadatum."""

        self.user_meta_repo.delete_metadatum(self.secret.id,
                                             remainder)
        msg = 'Deleted secret metadatum: %s for secret %s' % (remainder,
                                                              self.secret.id)
        pecan.response.status = 204
        LOG.info(msg)



# Copyright (c) 2016, GohighSec
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ast
import re
import six

import pep8


"""
Guidelines for writing new hacking checks

 - Use only for Barbican specific tests. OpenStack general tests
   should be submitted to the common 'hacking' module.
 - Pick numbers in the range B3xx. Find the current test with
   the highest allocated number and then pick the next value.
 - Keep the test method code in the source file ordered based
   on the B3xx value.
 - List the new rule in the top level HACKING.rst file
 - Add test cases for each new rule to barbican/tests/test_hacking.py

"""

oslo_namespace_imports = re.compile(r"from[\s]*oslo[.](.*)")
dict_constructor_with_list_copy_re = re.compile(r".*\bdict\((\[)?(\(|\[)")
assert_no_xrange_re = re.compile(r"\s*xrange\s*\(")
assert_True = re.compile(r".*assertEqual\(True, .*\)")
assert_None = re.compile(r".*assertEqual\(None, .*\)")
assert_Not_Equal = re.compile(r".*assertNotEqual\(None, .*\)")
assert_Is_Not = re.compile(r".*assertIsNot\(None, .*\)")
no_log_warn = re.compile(r".*LOG.warn\(.*\)")


class BaseASTChecker(ast.NodeVisitor):
    """Provides a simple framework for writing AST-based checks.

    Subclasses should implement visit_* methods like any other AST visitor
    implementation. When they detect an error for a particular node the
    method should call ``self.add_error(offending_node)``. Details about
    where in the code the error occurred will be pulled from the node
    object.

    Subclasses should also provide a class variable named CHECK_DESC to
    be used for the human readable error message.

    """

    CHECK_DESC = 'No check message specified'

    def __init__(self, tree, filename):
        """This object is created automatically by pep8.

        :param tree: an AST tree
        :param filename: name of the file being analyzed
                         (ignored by our checks)
        """
        self._tree = tree
        self._errors = []

    def run(self):
        """Called automatically by pep8."""
        self.visit(self._tree)
        return self._errors

    def add_error(self, node, message=None):
        """Add an error caused by a node to the list of errors for pep8."""
        message = message or self.CHECK_DESC
        error = (node.lineno, node.col_offset, message, self.__class__)
        self._errors.append(error)

    def _check_call_names(self, call_node, names):
        if isinstance(call_node, ast.Call):
            if isinstance(call_node.func, ast.Name):
                if call_node.func.id in names:
                    return True
        return False


class CheckLoggingFormatArgs(BaseASTChecker):
    """Check for improper use of logging format arguments.

    LOG.debug("Volume %s caught fire and is at %d degrees C and climbing.",
              ('volume1', 500))

    The format arguments should not be a tuple as it is easy to miss.

    """

    CHECK_DESC = 'B310 Log method arguments should not be a tuple.'
    LOG_METHODS = [
        'debug', 'info',
        'warn', 'warning',
        'error', 'exception',
        'critical', 'fatal',
        'trace', 'log'
    ]

    def _find_name(self, node):
        """Return the fully qualified name or a Name or Attribute."""
        if isinstance(node, ast.Name):
            return node.id
        elif (isinstance(node, ast.Attribute)
                and isinstance(node.value, (ast.Name, ast.Attribute))):
            method_name = node.attr
            obj_name = self._find_name(node.value)
            if obj_name is None:
                return None
            return obj_name + '.' + method_name
        elif isinstance(node, six.string_types):
            return node
        else:  # could be Subscript, Call or many more
            return None

    def visit_Call(self, node):
        """Look for the 'LOG.*' calls."""
        # extract the obj_name and method_name
        if isinstance(node.func, ast.Attribute):
            obj_name = self._find_name(node.func.value)
            if isinstance(node.func.value, ast.Name):
                method_name = node.func.attr
            elif isinstance(node.func.value, ast.Attribute):
                obj_name = self._find_name(node.func.value)
                method_name = node.func.attr
            else:  # could be Subscript, Call or many more
                return super(CheckLoggingFormatArgs, self).generic_visit(node)

            # obj must be a logger instance and method must be a log helper
            if (obj_name != 'LOG'
                    or method_name not in self.LOG_METHODS):
                return super(CheckLoggingFormatArgs, self).generic_visit(node)

            # the call must have arguments
            if not len(node.args):
                return super(CheckLoggingFormatArgs, self).generic_visit(node)

            # any argument should not be a tuple
            for arg in node.args:
                if isinstance(arg, ast.Tuple):
                    self.add_error(arg)

        return super(CheckLoggingFormatArgs, self).generic_visit(node)


class CheckForStrUnicodeExc(BaseASTChecker):
    """Checks for the use of str() or unicode() on an exception.

    This currently only handles the case where str() or unicode()
    is used in the scope of an exception handler.  If the exception
    is passed into a function, returned from an assertRaises, or
    used on an exception created in the same scope, this does not
    catch it.
    """

    CHECK_DESC = ('B314 str() and unicode() cannot be used on an '
                  'exception.  Remove or use six.text_type()')

    def __init__(self, tree, filename):
        super(CheckForStrUnicodeExc, self).__init__(tree, filename)
        self.name = []
        self.already_checked = []

    # Python 2
    def visit_TryExcept(self, node):
        for handler in node.handlers:
            if handler.name:
                self.name.append(handler.name.id)
                super(CheckForStrUnicodeExc, self).generic_visit(node)
                self.name = self.name[:-1]
            else:
                super(CheckForStrUnicodeExc, self).generic_visit(node)

    # Python 3
    def visit_ExceptHandler(self, node):
        if node.name:
            self.name.append(node.name)
            super(CheckForStrUnicodeExc, self).generic_visit(node)
            self.name = self.name[:-1]
        else:
            super(CheckForStrUnicodeExc, self).generic_visit(node)

    def visit_Call(self, node):
        if self._check_call_names(node, ['str', 'unicode']):
            if node not in self.already_checked:
                self.already_checked.append(node)
                if isinstance(node.args[0], ast.Name):
                    if node.args[0].id in self.name:
                        self.add_error(node.args[0])
        super(CheckForStrUnicodeExc, self).generic_visit(node)


def check_oslo_namespace_imports(logical_line, physical_line, filename):
    """'oslo_' should be used instead of 'oslo.'

    B317
    """
    if pep8.noqa(physical_line):
        return
    if re.match(oslo_namespace_imports, logical_line):
        msg = ("B317: '%s' must be used instead of '%s'.") % (
            logical_line.replace('oslo.', 'oslo_'),
            logical_line)
        yield(0, msg)


def dict_constructor_with_list_copy(logical_line):
    """Use a dict comprehension instead of a dict constructor

    B318
    """
    msg = ("B318: Must use a dict comprehension instead of a dict constructor"
           " with a sequence of key-value pairs."
           )
    if dict_constructor_with_list_copy_re.match(logical_line):
        yield (0, msg)


def no_xrange(logical_line):
    """Do not use 'xrange'

    B319
    """
    if assert_no_xrange_re.match(logical_line):
        yield(0, "B319: Do not use xrange().")


def validate_assertTrue(logical_line):
    """Use 'assertTrue' instead of 'assertEqual'

    B312
    """
    if re.match(assert_True, logical_line):
        msg = ("B312: Unit tests should use assertTrue(value) instead"
               " of using assertEqual(True, value).")
        yield(0, msg)


def validate_assertIsNone(logical_line):
    """Use 'assertIsNone' instead of 'assertEqual'

    B311
    """
    if re.match(assert_None, logical_line):
        msg = ("B311: Unit tests should use assertIsNone(value) instead"
               " of using assertEqual(None, value).")
        yield(0, msg)


def no_log_warn_check(logical_line):
    """Disallow 'LOG.warn'

    B320
    """
    msg = ("B320: LOG.warn is deprecated, please use LOG.warning!")
    if re.match(no_log_warn, logical_line):
        yield(0, msg)


def validate_assertIsNotNone(logical_line):
    """Use 'assertIsNotNone'

    B321
    """
    if re.match(assert_Not_Equal, logical_line) or \
       re.match(assert_Is_Not, logical_line):
        msg = ("B321: Unit tests should use assertIsNotNone(value) instead"
               " of using assertNotEqual(None, value) or"
               " assertIsNot(None, value).")
        yield(0, msg)


def factory(register):
    register(CheckForStrUnicodeExc)
    register(CheckLoggingFormatArgs)
    register(check_oslo_namespace_imports)
    register(dict_constructor_with_list_copy)
    register(no_xrange)
    register(validate_assertTrue)
    register(validate_assertIsNone)
    register(no_log_warn_check)
    register(validate_assertIsNotNone)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Server-side Keystone notification payload processing logic.
"""

from barbican.common import utils
from barbican import i18n as u
from barbican.model import repositories as rep
from barbican.tasks import resources


LOG = utils.getLogger(__name__)


class KeystoneEventConsumer(resources.BaseTask):
    """Event consumer listening for notifications sent by Keystone deployment.

    Currently this processes only Keystone project delete event.
    """

    def get_name(self):
        return u._('Project cleanup via Keystone notifications')

    def __init__(self, db_start=rep.start, db_commit=rep.commit,
                 db_rollback=rep.rollback, db_clear=rep.clear):
        LOG.debug('Creating KeystoneEventConsumer task processor')

        self.db_start = db_start
        self.db_commit = db_commit
        self.db_rollback = db_rollback
        self.db_clear = db_clear

    def process(self, *args, **kwargs):
        try:
            self.db_start()
            super(KeystoneEventConsumer, self).process(*args, **kwargs)
            self.db_commit()
        except Exception:
            """Exceptions that reach here needs to revert the entire
            transaction.
            No need to log error message as its already done earlier.
            """
            self.db_rollback()
            raise
        finally:
            self.db_clear()

    def retrieve_entity(self, project_id, resource_type=None,
                        operation_type=None):
        project_repo = rep.get_project_repository()
        return project_repo.find_by_external_project_id(
            external_project_id=project_id,
            suppress_exception=True)

    def handle_processing(self, barbican_project, *args, **kwargs):
        self.handle_cleanup(barbican_project, *args, **kwargs)

    def handle_error(self, project, status, message, exception,
                     project_id=None, resource_type=None, operation_type=None):
        LOG.error(
            'Error processing Keystone event, project_id=%(project_id)s, '
            'event resource=%(resource)s, event operation=%(operation)s, '
            'status=%(status)s, error message=%(message)s',
            {
                'project_id': project.project_id,
                'resource': resource_type,
                'operation': operation_type,
                'status': status,
                'message': message
            })

    def handle_success(self, project, result, project_id=None,
                       resource_type=None, operation_type=None):
        # Note: The processing 'result' argument can be ignored as 'result'
        # only pertains to long-running tasks. See the documentation for
        # BaseTask for more details.
        LOG.info(
            'Successfully handled Keystone event, '
            'project_id=%(project_id)s, event resource=%(resource)s, '
            'event operation=%(operation)s',
            {
                'project_id': project_id,
                'resource': resource_type,
                'operation': operation_type
            }
        )

    def handle_cleanup(self, project, project_id=None, resource_type=None,
                       operation_type=None):
        """Cleans up Barbican resources needed for Keystone project delete.

        :param project: Barbican project entity which is retrieved by project
        id available in Keystone notification.
        :param project_id: project identifier as present in Keystone
        notification.
        :param resource_type: type of resource updated as part of Keystone
        notification e.g. Keystone project, domain, user etc.
        :param operation_type: type of operation (created, updated, deleted
        etc.) performed on Keystone resource.
        """
        if project is None:
            LOG.info('No action is needed as there are no Barbican resources '
                     'present for Keystone project_id=%s', project_id)
            return

        # barbican entities use projects table 'id' field as foreign key.
        # Delete apis are using that id to lookup related entities and not
        # keystone project id which requires additional project table join.
        project_id = project.id

        rep.delete_all_project_resources(project_id)

        # reached here means there is no error so log the successful
        # cleanup log entry.
        LOG.info('Successfully completed Barbican resources cleanup for '
                 'Keystone project_id=%s', project_id)
